#ifndef AlienCommand_h
#define AlienCommand_h

// $Id: AlienCommand.h,v 1.17 2012/11/29 17:06:24 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "Counter.h"
#include "Debug.h"
#include "FileInfo.h"
#include "QOrderedSet.h"

#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QHash>
#include <QtCore/QList>

//! handles top commands
class AlienCommand: public Counter
{

    public:

    //! enumeration for command type
    enum Type
    {
        None,
        Connect,
        CheckProxy,
        FlushErrors,
        HomeDirectory,
        CreateDirectory,
        ShowDirectory,
        Copy,
        Locate,
        Rename,
        Remove,
        Open,
        Find
    };

    //! list of commands
    typedef QList<AlienCommand> List;

    //! constructor
    AlienCommand( const Type = None );

    //! destructor
    virtual ~AlienCommand( void )
    {}

    //! equal to operator
    virtual bool operator == ( const AlienCommand& other ) const
    { return id() == other.id(); }

    //! less than operator
    virtual bool operator < ( const AlienCommand& other ) const
    { return id() < other.id(); }

    //!@name accessors
    //@{

    //! command name
    QString name( void ) const
    { return name( type() ); }

    //! map Type to command name
    static QString name( const Type& type );

    //! id
    int id( void ) const
    { return id_; }

    //! type
    Type type( void ) const
    { return type_; }

    // true if arguments are passed
    bool hasArguments( void ) const
    { return !arguments_.isEmpty(); }

    //! true if argument is present
    bool hasArgument( int index ) const
    { return index < arguments_.size(); }

    //! argument size
    int argumentCount( void ) const
    { return arguments_.size(); }

    //! get argument
    const FileInfo& argument( int index ) const
    { return arguments_[index]; }

    //! return error message associated to this command, if failed
    QString errorString( void ) const;

    //! error details
    QString errorDetails( void ) const
    { return details_; }

    //@}

    //!@name modifiers
    //@{

    //! add argument
    AlienCommand& addArgument( const FileInfo& argument )
    {
        arguments_ << argument;
        return *this;
    }

    //! add arguments
    AlienCommand& addArguments( const FileInfo::List& arguments )
    {
        arguments_ << arguments;
        return *this;
    }

    //! push arguments
    AlienCommand& operator << (const FileInfo& argument )
    { return addArgument( argument ); }

    //! clear arguments
    void clearArguments( void )
    { arguments_.clear(); }

    //! set error details
    void setErrorDetails( const QString& value )
    { details_ = value; }

    //@}

    private:

    //! static id counter
    static unsigned int& _counter( void );

    //! map types to command name
    typedef QHash<Type,QString> NameMap;

    //! name map
    static NameMap& _names( void );

    //! initialize command names
    static void _initializeNames( void );

    //! unique id
    int id_;

    //! type
    Type type_;

    //! details (in case of errors)
    QString details_;

    //! arguments
    FileInfo::List arguments_;

};

//! used to sort commands based on and type
class SortedAlienCommand: public AlienCommand
{

    public:

    //! constructor
    SortedAlienCommand( void )
    {}

    //! constructor
    SortedAlienCommand( const AlienCommand& other ):
        AlienCommand( other )
    {}

    //! less than operator
    virtual bool operator < (const SortedAlienCommand& other ) const
    {

        if( type() != other.type() ) return type() < other.type();

        // check arguments
        for( int index = 0; index < std::min( argumentCount(), other.argumentCount() ); ++index )
        { if( argument(index).file() != other.argument(index).file() ) return argument(index).file() < other.argument(index).file(); }

        // check arguments size
        return argumentCount() < other.argumentCount();

    };

    typedef QOrderedSet<SortedAlienCommand> Set;

};

#endif
