// $Id: AlienCommandModel.cpp,v 1.9 2012/11/22 22:25:32 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "AlienCommandModel.h"

#include "CustomPixmap.h"
#include "Icons.h"
#include "IconEngine.h"
#include "IconSize.h"
#include "TimeStamp.h"
#include "XmlOptions.h"

#include <QIcon>

//_______________________________________________
const QString AlienCommandModel::columnTitles_[ AlienCommandModel::nColumns ] =
{ "Name", "File" };

//_______________________________________________________________
AlienCommandModel::AlienCommandModel( QObject* parent ):
    ListModel<AlienCommand>( parent ),
    Counter( "AlienCommandModel" )
{

    Debug::Throw( "AlienCommandModel::AlienCommandModel.\n" );
    updateConfiguration();

}

//__________________________________________________________________
QVariant AlienCommandModel::data( const QModelIndex& index, int role ) const
{

    // check index, role and column
    if( !index.isValid() ) return QVariant();

    // retrieve associated command
    const AlienCommand& command( get()[index.row()] );

    // return text associated to command and column
    if( role == Qt::DisplayRole ) {

        switch( index.column() )
        {
            case NAME: return command.name();
            case FILE: return command.argumentCount() > 0  ? command.argument(0).file():QVariant();
            default: return QVariant();
        }
    }

    // return icon associated to file
    if( role == Qt::DecorationRole && index.column() == NAME )
    { return _icon(command.type()); }

    return QVariant();

}

//__________________________________________________________________
QVariant AlienCommandModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    if(
        orientation == Qt::Horizontal &&
        role == Qt::DisplayRole &&
        section >= 0 &&
        section < nColumns )
    { return columnTitles_[section]; }

    // return empty
    return QVariant();

}

//_________________________________________________________
void AlienCommandModel::updateConfiguration( void )
{

    Debug::Throw( "AlienCommandModel::updateConfiguration.\n" );

    // install pixmaps
    _icons().clear();
    _installIcons();

    reset();

}

//_____________________________________________________________________
void AlienCommandModel::_installIcons( void ) const
{

    Debug::Throw( "AlienCommandList::_installIcons" );

    if( !_icons().empty() ) return;

    // type icons
    typedef QHash< AlienCommand::Type, QString > IconNames;
    IconNames typeNames;
    typeNames[AlienCommand::Connect] = ICONS::CONNECT;
    typeNames[AlienCommand::CheckProxy] = ICONS::CONNECT;
    typeNames[AlienCommand::HomeDirectory] = ICONS::HOME;
    typeNames[AlienCommand::CreateDirectory] = ICONS::CREATE_DIRECTORY;
    typeNames[AlienCommand::ShowDirectory] = ICONS::FIND;
    typeNames[AlienCommand::Copy] = ICONS::PASTE;
    typeNames[AlienCommand::Rename] = ICONS::RENAME;
    typeNames[AlienCommand::Remove] = ICONS::DELETE;
    typeNames[AlienCommand::Open] = ICONS::EDIT;
    typeNames[AlienCommand::Find] = ICONS::FIND;

    // load link pixmap
    for( IconNames::iterator iter = typeNames.begin(); iter != typeNames.end(); ++iter )
    { _icons().insert( iter.key(), IconEngine::get( iter.value() ) ); }

    return;
}

//_______________________________________________
AlienCommandModel::IconCache& AlienCommandModel::_icons( void )
{
    static IconCache cache;
    return cache;
}

//____________________________________________________________
const QIcon& AlienCommandModel::_icon( AlienCommand::Type type )
{

    IconCache::iterator iter = ( _icons().find( type ) );
    if( iter != _icons().end() ) return iter.value();
    else return _icons().insert( type, CustomPixmap().empty( IconSize( IconSize::Medium ) ) ).value();

}

//________________________________________________________
bool AlienCommandModel::SortFTor::operator () ( AlienCommand first, AlienCommand second ) const
{

    if( order_ == Qt::DescendingOrder ) std::swap( first, second );
    switch( type_ )
    {

        case AlienCommandModel::NAME:
        if( first.type() != second.type() ) return first.type() < second.type();
        else break;

        case AlienCommandModel::FILE:
        {
            const FileInfo source( first.argumentCount() > 0 ? first.argument(0).file():FileInfo() );
            const FileInfo destination( second.argumentCount() > 0 ? second.argument(0).file():FileInfo() );
            if( source.file() != destination.file() ) return source.file() < destination.file();
            else break;
        }

        default: break;
    }

    return first.id() < second.id();

}
