// $Id: AlienInterface.cpp,v 1.71 2012/11/30 10:28:00 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "AlienInterface.h"

#include "Application.h"
#include "CustomProcess.h"
#include "Command.h"
#include "Debug.h"
#include "File.h"
#include "IOString.h"
#include "Singleton.h"
#include "TextFormat.h"
#include "Util.h"
#include "XmlOptions.h"

#include <QtCore/QRegExp>
#include <QtCore/QTextStream>
#include <QtCore/QTimer>
#include <algorithm>

//_______________________________________________
AlienInterface::AlienInterface( QObject *parent ):
    QObject( parent ),
    Counter( "AlienInterface" ),
    showHiddenFiles_( false ),
    process_( 0 ),
    thread_( this ),
    stdoutIndex_( 0 ),
    stderrIndex_( 0 ),
    wasConnected_( false ),
    connected_( true ),
    total_( 1 ),
    transfered_( 0 ),
    currentTransfered_( 0 ),
    replica_( 1 ),
    maxReplica_( 1 )
{
    Debug::Throw("AlienInterface::AlienInterface.\n");
    Application& application( *static_cast<Application*>(Singleton::get().application()) );
    connect( &application, SIGNAL( configurationChanged() ), SLOT( _updateConfiguration() ) );

    // thread connections
    connect( &thread_, SIGNAL( filesAvailable( const File::List& ) ), SLOT( _processFiles( const File::List& ) ) );
    connect( &thread_, SIGNAL( sizeAvailable( qint64 ) ), SLOT( _processSize( qint64 ) ) );
    connect( &thread_, SIGNAL( finished( void ) ), SLOT( _commandCompleted() ) );

    // configuration
    _updateConfiguration();
}

//_______________________________________________
AlienInterface::~AlienInterface( void )
{ Debug::Throw("AlienInterface::~AlienInterface.\n"); }

//_______________________________________________
void AlienInterface::findCertificates( void )
{

    Debug::Throw( "AlienInterface::findCertificates.\n" );

    // check if option is already set and valid
    if( XmlOptions::get().contains( "CERTIFICATES_PATH" ) &&
        File( XmlOptions::get().raw( "CERTIFICATES_PATH" ) ).exists() &&
        File( XmlOptions::get().raw( "CERTIFICATES_PATH" ) ).isAbsolute() )
    { return; }

    File alienPath;

    // try load from gbbox path
    if( XmlOptions::get().contains( "GBBOX" ) && File( XmlOptions::get().raw( "GBBOX" ) ).isAbsolute() )
    {
        File gbboxPath( File( XmlOptions::get().raw( "GBBOX" ) ).path() );
        alienPath = File( "../.." ).addPath( gbboxPath ).canonicalName();
    }

    if( !alienPath.exists() )
    {

        // try load using GSHELL_ROOT
        const QProcessEnvironment environment( QProcessEnvironment::systemEnvironment() );
        if( environment.contains( "GSHELL_ROOT" ) )
        { alienPath = File( ".." ).addPath( environment.value( "GSHELL_ROOT" ) ).canonicalName(); }

        // try load using ALIEN_ROOT environment
        if( !alienPath.exists() && environment.contains( "ALIEN_ROOT" ) )
        { alienPath = File( environment.value( "ALIEN_ROOT" ) ).canonicalName(); }

    }

    if( !alienPath.exists() ) return;

    // possible path for certificates, relative to alien path
    File::List pathList;
    pathList
        << File( "globus/share/certificates" )
        << File( "api/share/certificates" );

    // get certificates path
    File certificatesPath;
    foreach( const File& path, pathList )
    {
        const File tmp( path.addPath( alienPath ).canonicalName() );
        if( tmp.exists() )
        {
            certificatesPath = tmp;
            break;
        }
    }

    // assign
    if( certificatesPath.exists() )
    { XmlOptions::get().setRaw( "CERTIFICATES_PATH", certificatesPath ); }

    return;

}

//__________________________________________________________________________________
void AlienInterface::addMacro( const AlienMacro& macro )
{

    Debug::Throw() << "AlienInterface::addMacro - " << macro.name() << endl;

    // check if empty
    const bool empty( macros_.empty() );

    // add macro
    macros_ << macro;

    // emit signal
    emit macroAdded( macro );

    // run macro if there is none pending
    if( empty )
    {
        emit busyStateChanged( true );
        QTimer::singleShot( 0, this, SIGNAL( busy( void ) ) );
        QTimer::singleShot( 0, this, SLOT( _runMacro( void ) ) );
    }

}

//__________________________________________________________________________________
void AlienInterface::addMacros( const AlienMacro::List& macros )
{

    Debug::Throw( "AlienInterface::addMacros.\n" );

    const bool empty( macros_.empty() );
    foreach( const AlienMacro& macro, macros )
    { macros_ << macro; }

    // check if macros have been effectively added
    emit macrosAdded( macros );
    if( empty )
    {
        emit busyStateChanged( true );
        QTimer::singleShot( 0, this, SIGNAL( busy( void ) ) );
        QTimer::singleShot( 0, this, SLOT( _runMacro( void ) ) );
    }

}

//____________________________________________________________
void AlienInterface::abortAll( void )
{
    Debug::Throw( "AlienInterface::abortAll.\n" );

    // remove all macros from list but the current
    if( macros_.size() > 1 )
    {
        for( AlienMacro::List::iterator iter = ( ++macros_.begin() ); iter != macros_.end(); ++iter )
        { emit macroCompleted( *iter ); }

        macros_.erase( ++macros_.begin(), macros_.end() );
    }

    abortCurrent();

}

//____________________________________________________________
void AlienInterface::abortCurrent( void )
{

    Debug::Throw( "AlienInterface::abortCurrent.\n" );

    // remove all commands from list but the current
    if( commands_.size() > 1 )
    {
        for( AlienCommand::List::iterator iter = ( ++commands_.begin() ); iter != commands_.end(); ++iter )
        { emit commandCompleted( *iter ); }

        commands_.erase( ++commands_.begin(), commands_.end() );
    }

    // kill process if any. Should terminate current command and process automatically
    if( process_ && process_->state() != QProcess::NotRunning ) {

        Debug::Throw() << "AlienInterface::abortCurrent - killing" << endl;
        process_->kill();

    } else {

        Debug::Throw() << "AlienInterface::abortCurrent - clearing" << endl;
        commands_.clear();
        if( !macros_.empty() ) _macroCompleted();

    }

}

//__________________________________________________________________________________
void AlienInterface::_runMacro( void )
{

    // do nothing if no macros are in the pile
    if( macros_.empty() )
    {
        QTimer::singleShot( 0, this, SIGNAL( ready( void ) ) );
        emit busyStateChanged( false );
        return;
    }

    // front command
    AlienMacro macro( macros_.front() );
    emit macroStarted( macro );
    Debug::Throw() << "AlienInterface::_runMacro - macro: " << macro.name() << " (" << macro.id() << ")" << endl;

    // take action depending on macro type
    switch( macro.type() )
    {

        case AlienMacro::Add:
        {
            foreach( const FileInfo& fileInfo, macro.arguments() )
            { _addFileRecursive( fileInfo ); }

            _macroCompleted();
            break;
        }

        case AlienMacro::Clear:
        {
            _clearFilesRecursive();
            _clearFiles();
            clearFailedCommands();
            _macroCompleted();
            break;
        }

        case AlienMacro::FlushErrors:
        {

            _addCommand( AlienCommand::FlushErrors );
            break;

        }


        case AlienMacro::Connect:
        {
            _addCommand( AlienCommand( AlienCommand::Connect ).addArguments( macro.arguments() ) );
            break;
        }

        case AlienMacro::CheckProxy:
        {
            _addCommand( AlienCommand( AlienCommand::CheckProxy ).addArguments( macro.arguments() ) );
            break;
        }

        case AlienMacro::HomeDirectory:
        {
            // skip command if another one with same type is found in the list
            bool found = false;
            for( int i = 1; i < macros_.size() && !found; ++i )
            {
                const AlienMacro& other( macros_[i] );
                found |= ( macro.type() == other.type() && macro.argument(0).type() == other.argument(0).type() );
            }

            if( !found ) _addCommand( AlienCommand( AlienCommand::HomeDirectory ).addArguments( macro.arguments() ) );
            else _macroCompleted();

            break;
        }

        case AlienMacro::ShowDirectory:
        {

            // skip command if another one with same type is found in the list
            bool found = false;
            for( int i = 1; i < macros_.size() && !found; ++i )
            {
                const AlienMacro& other( macros_[i] );
                found |= ( macro.type() == other.type() && macro.argument(0).type() == other.argument(0).type() );
            }

            if( !found ) _addCommand( AlienCommand( AlienCommand::ShowDirectory ).addArguments( macro.arguments() ) );
            else _macroCompleted();

            break;

        }

        case AlienMacro::CreateDirectory:
        {
            _addCommand( AlienCommand( AlienCommand::CreateDirectory ).addArguments( macro.arguments() ) );
            break;
        }

        case AlienMacro::Rename:
        {
            emit macroStarted( macro );
            _addCommand( AlienCommand( AlienCommand::Rename ).addArguments( macro.arguments() ) );
            break;
        }

        case AlienMacro::Remove:
        {
            _addCommand( AlienCommand( AlienCommand::Remove ).addArguments( macro.arguments() ) );
            break;
        }

        case AlienMacro::Copy:
        {
            _addCommand( AlienCommand( AlienCommand::Copy ).addArguments( macro.arguments() ) );
            break;
        }

        case AlienMacro::Locate:
        {
            _addCommand( AlienCommand( AlienCommand::Locate ).addArguments( macro.arguments() ) );
            break;
        }

        case AlienMacro::CopyRecursive:
        {

            // compute total size to transfer
            _computeTotalSize();
            emit maxProgressAvailable( 100 );
            emit progressAvailable( 0 );

            // get path from arguments
            const FileInfo sourcePath( macro.argument(0) );
            const FileInfo destinationPath( macro.argument(1) );

            // create list of commands
            AlienCommand::List commands;

            // loop over files recursive
            foreach( FileInfo source, filesRecursive_ )
            {

                // create destination, by substituting destinationPath to sourcePath
                FileInfo destination( source );
                File newPath( source.file() );
                _removePath( newPath, sourcePath.file() );
                destination.setFile( newPath.addPath( destinationPath.file() ) );

                // set proper location flag
                if( destinationPath.isLocal() ) destination.setLocal();
                else destination.setRemote();

                // when copying link to remote destination, use real file
                if( source.isLocal() && source.isLink() && destination.isRemote() )
                {
                    if( source.isBrokenLink() )
                    {

                        QString buffer;
                        QTextStream( &buffer ) << "File \"" << source.file() << "\" is a broken link. <copy> aborted." << endl;
                        emit logAvailable( buffer );
                        continue;

                    } else {

                        // use real file, update source accordingly
                        source.setFile( source.file().readLink() );
                        source.setIsLink( false );
                        source.update();

                        destination.setIsLink( false );
                        destination.updateFrom( source );

                    }
                }

                // add proper command
                // links are copied unchanged, when destination is local
                if( destination.isDocument() || (destination.isLink() && destination.isLocal() ) ) commands << ( AlienCommand( AlienCommand::Copy ) << source << destination );
                else commands << (AlienCommand( AlienCommand::CreateDirectory ) << destination );

            }

            // submit commands
            if( !commands.isEmpty() ) _addCommands( commands );
            else _macroCompleted();

            break;

        }

        case AlienMacro::RemoveRecursive:
        {

            // compute total size to transfer
            _computeTotalSize();
            emit maxProgressAvailable( 100 );
            emit progressAvailable( 0 );

            // loop through files recursive and properly propagates failure flags
            FileInfo::List failed;
            foreach( const FileInfo& fileInfo, filesRecursive_ )
            { if( fileInfo.isFailed() ) failed << fileInfo; }

            for( FileInfo::List::iterator iter = filesRecursive_.begin(); iter != filesRecursive_.end(); iter++ )
            {
                if( iter->isFailed() ) continue;

                // see if current is a base directory of a failed file
                // if so, tag as failed as well
                foreach( const FileInfo& fileInfo, failed )
                { if( fileInfo.file().startsWith( iter->file() ) ) iter->setFailed(); }
            }

            // create list of commands
            AlienCommand::List commands;

            // loop backwards through filesRecursive and emit remove command for non failed jobs
            FileInfo::ListIterator iter( filesRecursive_ );
            iter.toBack();
            while( iter.hasPrevious() )
            {
                const FileInfo& fileInfo( iter.previous() );
                if( !fileInfo.isFailed() ) commands << ( AlienCommand( AlienCommand::Remove ) << fileInfo );
                else {

                    QString buffer;
                    QTextStream( &buffer ) << fileInfo.file() << " was not properly copied. <remove> canceled.";
                    emit logAvailable( buffer );

                }

            }

            // submit commands
            if( !commands.isEmpty() ) _addCommands( commands );
            else _macroCompleted();

            break;

        }

        case AlienMacro::Open:
        {
            _addCommand( AlienCommand( AlienCommand::Open ).addArguments( macro.arguments() ) );
            break;
        }

        case AlienMacro::Find:
        {
            _addCommand( AlienCommand( AlienCommand::Find ).addArguments( macro.arguments() ) );
            break;
        }

        case AlienMacro::SizeRecursive:
        case AlienMacro::ListRecursive:
        {

            if( macro.type() == AlienMacro::SizeRecursive ) _resetTotalSize();

            // append current directory
            const FileInfo path( macro.argument(0) );
            _addFileRecursive( path );

            // add list command if folder
            // skip soft links if ListRecursive targets a local directory
            if( !path.isFolder() )
            {

                _macroCompleted();

            } else {

                _addCommand( AlienCommand( AlienCommand::ShowDirectory ).addArguments( macro.arguments() ) );

            }

            break;
        }

        default:
        _macroCompleted();
        break;

    }

    return;

}

//______________________________________________________________
void AlienInterface::_macroCompleted( bool )
{

    Debug::Throw( "AlienInterface::_macroCompleted.\n" );

    if( !macros_.empty() )
    {

        AlienMacro macro( macros_.front() );
        switch( macro.type() )
        {

            case AlienMacro::ShowDirectory:
            {
                emit contentAvailable( macro.argument(0), filesRecursive_ );
                break;
            }

            case AlienMacro::SizeRecursive:
            if( macro.argument(0).isRemote() )
            {
                _computeTotalSize();
                emit sizeAvailable( macro.argument(0), total_ );
            }
            break;

            default: break;

        }

        emit macroCompleted( macro );
        macros_.removeFirst();

    }

    _runMacro();

}

//______________________________________________________________
void AlienInterface::_commandCompleted( int returnCode, QProcess::ExitStatus status )
{

    /*
    if there is no current command in the queue (in case of abort)
    just jump to the next macro
    */
    if( commands_.empty() )
    {
        _macroCompleted();
        return;
    }

    // get current command
    AlienCommand command( commands_.front() );

    // debug output
    Debug::Throw() << "AlienInterface::_commandCompleted - command: " << command.name() << " (" << command.id() << ") code: " << returnCode << endl;

    // get current macro (if any)
    AlienMacro macro( macros_.isEmpty() ? AlienMacro():macros_.front() );

    // check connection from stderr buffer
    bool checkConnection( false );
    if( !macro.hasFlag( AlienMacro::IgnoreDisconnect ) )
    {
        switch( command.type() )
        {
            case AlienCommand::Connect:
            case AlienCommand::Find:
            {
                checkConnection = true;
                break;
            }

            case AlienCommand::ShowDirectory:
            case AlienCommand::CreateDirectory:
            case AlienCommand::HomeDirectory:
            case AlienCommand::Locate:
            case AlienCommand::Remove:
            {
                if( command.argument(0).isRemote() )
                { checkConnection = true; }

                break;
            }

            case AlienCommand::Copy:
            case AlienCommand::Rename:
            {
                if( command.argument(0).isRemote() || command.argument(1).isRemote() )
                { checkConnection = true; }
                break;
            }

            default: break;

        }

    }

    if( checkConnection ) { _checkConnection(); }

    // process errors, if any
    if( status != QProcess::NormalExit || ( _hasProcess() && _process().error() == QProcess::FailedToStart ) || returnCode != 0 )
    {

        QString buffer;
        QTextStream( &buffer ) << "Failed to " << command.name();
        emit messageAvailable( buffer );
        emit logAvailable( buffer + "\n\n", FORMAT::Italic|FORMAT::Bold );

        // if found, mark the relevant file in "filesRecursive" as failed
        if( macro.type() == AlienMacro::CopyRecursive && command.hasArgument(0) )
        {
            FileInfo::List::iterator iter( std::find( filesRecursive_.begin(), filesRecursive_.end(), command.argument(0) ) );
            if( iter != filesRecursive_.end() ) iter->setFailed();
        }

        // store failed command
        command.setErrorDetails( stderrBuffer_.text() );
        failedCommands_.append( command );

        // add FlushError command
        if(
            !( macro.hasFlag( AlienMacro::IgnoreErrors ) || _ignoreError( command, stderrBuffer_.text() ) ) &&
            ( commands_.isEmpty() || commands_.back().type() != AlienCommand::FlushErrors ) )
        { _addCommand( AlienCommand::FlushErrors ); }

        // advance to next
        emit commandCompleted( command );
        commands_.removeFirst();

        _runCommand();
        return;

    }

    // add (cosmetics) empty line to log.
    emit logAvailable( "\n" );

    // process command
    switch( command.type() )
    {

        case AlienCommand::CheckProxy:
        {
            _processProxyInfo();
            break;
        }

        case AlienCommand::HomeDirectory:
        {

            Q_ASSERT( command.argument(0).isRemote() );
            const bool alsoUpdate( command.argumentCount() == 1 );
            _processHomeDirectory( alsoUpdate );
            break;
        }

        case AlienCommand::ShowDirectory:
        {

            // process directory
            if( command.argument(0).isRemote() )
            {

                _processDirectoryContent( command.argument(0) );

                // add files to recursive list
                FileInfo::List directories;
                foreach( const FileInfo& fileInfo, files_ )
                {
                    _addFileRecursive( fileInfo );

                    /*
                    add folders to list of directories;
                    skip soft links if ListRecursive targets a local directory
                    or if command is SizeRecursive
                    */
                    if( ( macro.type() == AlienMacro::ListRecursive &&
                        fileInfo.isFolder() && ( !fileInfo.isLink() || !macro.hasArgument(1) || macro.argument(1).isRemote() ) ) ||
                        ( macro.type() == AlienMacro::SizeRecursive && fileInfo.isFolder() && !fileInfo.isLink() ) )
                    { directories << fileInfo; }
                }

                // update total size
                if( macro.type() == AlienMacro::SizeRecursive )
                { if( _updateTotalSize() ) emit sizeAvailable( macro.argument(0), total_ ); }

                // clear files
                _clearFiles();

                // submit show directory commands if ListRecursive
                if( macro.type() == AlienMacro::ListRecursive || macro.type() == AlienMacro::SizeRecursive )
                {
                    foreach( const FileInfo& fileInfo, directories )
                    { _addCommand( AlienCommand( AlienCommand::ShowDirectory ) << fileInfo ); }
                }

            }

            break;
        }

        case AlienCommand::CreateDirectory:
        {
            emit fileInfoAdded( command.argument(0) );
            break;
        }

        case AlienCommand::Locate:
        {
            _processLocation( command.argument(0) );
            break;
        }

        case AlienCommand::Rename:
        {
            emit fileInfoRemoved( command.argument(0) );
            emit fileInfoAdded( command.argument(1) );
            break;
        }

        case AlienCommand::Remove:
        {
            emit fileInfoRemoved( command.argument(0) );
            if( macro.type() == AlienMacro::RemoveRecursive )
            {
                _updateTransferedSize( command.argument(0) );
                emit progressAvailable( (100.0*transfered_)/total_ );
            }
            break;
        }

        case AlienCommand::Copy:
        {

            const FileInfo source( command.argument(0) );
            const FileInfo destination( command.argument(1) );

            // check status
            if( source.isLocal() && destination.isLocal() && thread_.hasError() )
            {
                failedCommands_.append( command );
                if( ( commands_.isEmpty() || commands_.back().type() != AlienCommand::FlushErrors ) )
                { _addCommand( AlienCommand::FlushErrors ); }
            }

            emit fileInfoAdded( destination );
            if( macro.type() == AlienMacro::CopyRecursive )
            {
                _updateTransferedSize( command.argument(0) );
                emit progressAvailable( (100.0*transfered_)/total_ );
            }
            break;
        }

        case AlienCommand::Find:
        {
            emit collectionAvailable( stdoutBuffer_.text() );
            break;
        }

        default: break;

    }

    // emit signal, remove from pile and run next command if any
    emit commandCompleted( command );
    commands_.removeFirst();
    _runCommand();

    return;

}

//____________________________________________________________
void AlienInterface::_error( QProcess::ProcessError error )
{

    Q_ASSERT( !commands_.empty() );
    AlienCommand command( commands_.front() );
    Debug::Throw() << "AlienInterface::_error -"
        << " command: " << command.name()
        << " (" << command.id() << ")"
        << " error: " << error
        << endl;

    if( error == QProcess::FailedToStart )
    {

        /*
        in case the process failed to start, _commandComppleted is not called
        one needs to call it manually in order to proceed with command list
        */
        _commandCompleted( 1, QProcess::NormalExit );

    } else {

        QString buffer;
        QTextStream( &buffer ) << "Failed to " << command.name();
        emit messageAvailable( buffer );
        emit logAvailable( buffer+"\n\n", FORMAT::Italic|FORMAT::Bold );

    }

    return;
}

//___________________________________________________________________
void AlienInterface::_readStdout(  void )
{
    _process().setReadChannel( QProcess::StandardOutput );
    IOString buffer( _process() );

    // replace dummy characters
    {
        const QRegExp regExp( "(\\S){2,2}((\\d{2,2};\\d{2,2})|(0))m" );
        buffer.replace( regExp, "" );
    }

    emit logAvailable( buffer );
    stdoutBuffer_.append( buffer );

    // get replica
    // get current command
    const AlienCommand command = (commands_.isEmpty() ? AlienCommand():commands_.front() );
    if( command.type() == AlienCommand::Copy )
    {

        int position(0);
        QRegExp regExp( "Creating replica (\\d+)/(\\d+)" );
        while( (position = stdoutBuffer_.text().indexOf( regExp, stdoutIndex_ )) >= 0 )
        {
            replica_ = regExp.cap(1).toInt();
            maxReplica_ = regExp.cap(2).toInt();
            stdoutIndex_ = position + regExp.matchedLength();
        }

    }
}

//___________________________________________________________________
void AlienInterface::_readStderr(  void )
{
    _process().setReadChannel( QProcess::StandardError );
    IOString buffer( _process() );

    emit logAvailable( buffer, FORMAT::Italic );
    stderrBuffer_.append( buffer );

    // parse progress
    // get current command
    const AlienCommand command = (commands_.isEmpty() ? AlienCommand():commands_.front() );
    if( command.type() == AlienCommand::Copy )
    {
        // get progress
        qreal progress( -1 );
        int position(0);
        QRegExp regExp( "\\|(?:=*)(?:>\\.*)?\\|\\s+(\\d+(?:\\.\\d+)?)\\s+%" );
        while( (position = stderrBuffer_.text().indexOf( regExp, stderrIndex_ )) >= 0 )
        {
            progress = regExp.cap(1).toDouble();
            stderrIndex_ = position + regExp.matchedLength();
        }

        const qint64 currentSize( command.argument(0).size() );
        const qint64 transfered( (100*(replica_ - 1 ) + progress)*currentSize/100/maxReplica_ );
        if( transfered > currentTransfered_ )
        {
            currentTransfered_ = transfered;
            emit progressAvailable( (100.0*(transfered_ + currentTransfered_))/total_ );
        }
    }

}

//___________________________________________________________________________
void AlienInterface::_processFiles( const File::List& files )
{

    // get current macro
    if( macros_.empty() ) return;
    AlienMacro macro( macros_.front() );

    switch( macro.type() )
    {

        case AlienMacro::ShowDirectory:
        case AlienMacro::ListRecursive:
        foreach( const File& file, files )
        {

            // check if hidden
            if( !_showHiddenFiles() && file.isHidden() ) continue;

            // propagate file properties to FileInfo
            FileInfo fileInfo( file );
            if( file.isDirectory() ) fileInfo.setIsFolder();
            else fileInfo.setIsDocument();

            if( file.isLink() ) fileInfo.setIsLink();
            if( file.isBrokenLink() ) fileInfo.setIsBrokenLink();
            if( file.isHidden() ) fileInfo.setIsHidden();

            fileInfo.setSize( file.fileSize() );
            fileInfo.setLastModified( file.lastModified() );
            fileInfo.setUser( file.userName() );
            fileInfo.setGroup( file.groupName() );
            fileInfo.setPermissions( file.permissions() );

            _addFileRecursive( fileInfo );

            QString buffer;
            QTextStream what( &buffer );
            what << fileInfo.permissionsString()
                << " " << fileInfo.user()
                << " "
                << left << qSetFieldWidth(18) << fileInfo.group()
                << qSetFieldWidth(0) << fileInfo.size() << " "
                << qSetFieldWidth( 24 ) << TimeStamp( fileInfo.lastModified() ).toString()
                << qSetFieldWidth( 0 ) << fileInfo.file()
                << endl;
            emit logAvailable( buffer );
        }

        // cosmetics
        emit logAvailable( "\n" );

        break;

        default: break;

    }
}

//___________________________________________________________________________
void AlienInterface::_processSize( qint64 size )
{

    // get current macro
    if( macros_.empty() ) return;
    AlienMacro macro( macros_.front() );
    if( macro.type() == AlienMacro::SizeRecursive )
    { emit sizeAvailable( macro.argument(0), size ); }

}

//__________________________________________________________________________________
void AlienInterface::_addCommand( const AlienCommand& command )
{

    Debug::Throw() << "AlienInterface::_addCommand - " << command.name() << endl;

    // add command
    const bool empty( commands_.empty() );
    commands_ << command;

    // emit signal
    emit commandAdded( command );

    // run command if there is none pending
    if( empty ) _runCommand();

}

//__________________________________________________________________________________
void AlienInterface::_addCommands( const AlienCommand::List& commands )
{

    Debug::Throw( "AlienInterface::_addCommands.\n" );

    const bool empty( commands_.empty() );
    foreach( const AlienCommand& command, commands )
    { commands_ << command; }

    // check if commands have been effectively added
    emit commandsAdded( commands );
    if( empty ) _runCommand();

}


//__________________________________________________________________________________
void AlienInterface::_runCommand( void )
{

    // do nothing if no commands are in the pile
    if( commands_.empty() )
    {
        _macroCompleted();
        return;
    }

    // front command
    AlienCommand command( commands_.front() );
    Debug::Throw() << "AlienInterface::_runCommand - command: " << command.name() << " (" << command.id() << ")" << endl;

    // reset buffer
    _resetBuffers();
    _resetProcess();

    // retrieve current macro (if any)
    AlienMacro macro( macros_.isEmpty() ? AlienMacro():macros_.front() );

    // take action depending on command type
    switch( command.type() )
    {

        case AlienCommand::CheckProxy:
        {

            emit logAvailable( "Checking GSI proxy\n", FORMAT::Bold );
            QStringList arguments;
            arguments
                << XmlOptions::get().raw("GSI_PROXY" )
                << "info";

            _newProcess().start( arguments );
            emit commandStarted( command );
            return;

        }

        case AlienCommand::Connect:
        {

            emit logAvailable( "Trying to connect to remote filesystem\n", FORMAT::Bold );
            QStringList arguments;
            arguments
                << XmlOptions::get().raw("GBBOX" )
                << "connect"
                << XmlOptions::get().raw( "ALIEN_HOST" )
                << QString().setNum( XmlOptions::get().get<int>( "ALIEN_PORT" ) )
                << XmlOptions::get().raw( "ALIEN_USER" );

            _newProcess().start( arguments );

            return;

        }

        case AlienCommand::FlushErrors:
        {

            if( !failedCommands_.isEmpty() ) emit commandsFailed( failedCommands_ );
            if( !failedCommands_.isEmpty() )
            {

                emit logAvailable( "Trying to recover failed commands\n", FORMAT::Bold );

                AlienCommand::List commands( failedCommands_ );
                clearFailedCommands();
                _addCommands( commands );
                _addCommand( AlienCommand::FlushErrors );
            }

            break;

        }

        case AlienCommand::HomeDirectory:
        {

            emit logAvailable( "Requesting home directory\n", FORMAT::Bold );
            emit messageAvailable( "Requesting home directory" );

            if( command.argument(0).isLocal() )
            {

                const bool alsoUpdate( command.argumentCount() == 1 );
                emit homeAvailable( FileInfo( Util::home(), FileInfo::Folder ), alsoUpdate );
                emit logAvailable( Util::home()+"\n\n" );
                break;

            } else {

                QStringList arguments;
                arguments
                    << XmlOptions::get().raw("GBBOX" )
                    << "whoami";

                // run and emit signal
                _newProcess().start( arguments );
                emit commandStarted( command );
                return;

            }

        }

        case AlienCommand::ShowDirectory:
        {

            // normal processing
            const FileInfo& path( command.argument(0) );
            QString buffer;
            QTextStream( &buffer ) << "Requesting directory content \"" << path.file() << "\"" << endl;
            emit logAvailable( buffer, FORMAT::Bold );
            emit messageAvailable( "Requesting directory content" );
            emit commandStarted( command );

            if( path.isLocal() )
            {

                // setup thread
                thread_.setFile( path.file() );

                if( macro.type() == AlienMacro::ShowDirectory ) thread_.setCommand( FileThread::List );
                else if( macro.type() == AlienMacro::SizeRecursive ) thread_.setCommand( FileThread::SizeRecursive );
                else if( macro.type() == AlienMacro::ListRecursive ) thread_.setCommand( FileThread::ListRecursive );

                File::ListFlags flags( File::None );
                if( macro.type() == AlienMacro::ListRecursive || macro.type() == AlienMacro::SizeRecursive )
                { flags |= File::Recursive; }

                if( _showHiddenFiles() || macro.type() == AlienMacro::SizeRecursive )
                { flags |= File::ShowHiddenFiles; }

                if( macro.type() == AlienMacro::ListRecursive && macro.hasArgument(1) && macro.argument(1).isRemote() )
                { flags |= File::FollowLinks; }

                thread_.setFlags( flags );
                thread_.start();
                return;

            } else {

                // build command
                QStringList arguments;
                arguments
                    << XmlOptions::get().raw( "GBBOX" )
                    << "ls" << "-n" << "-l" << path.file();

                // run and emit signal
                _newProcess().start( arguments );
                return;

            }

        }

        case AlienCommand::CreateDirectory:
        {

            const FileInfo& path( command.argument(0) );
            QString buffer;
            QTextStream( &buffer ) << "Creating new folder \"" << path.file() << "\"" << endl;
            emit logAvailable( buffer, FORMAT::Bold );
            emit messageAvailable( "Creating new folder" );

            if( path.isLocal() )
            {

                if( !path.file().createDirectory() )
                {

                    QString buffer;
                    QTextStream( &buffer ) << "Failed to create folder: " << path.file() << "." << endl;
                    emit logAvailable( buffer );

                    // log failed command
                    command.setErrorDetails( buffer );
                    failedCommands_.append( command );
                    if( ( commands_.isEmpty() || commands_.back().type() != AlienCommand::FlushErrors ) )
                    { _addCommand( AlienCommand::FlushErrors ); }

                } else emit fileInfoAdded( path );
                break;

            } else {

                QStringList arguments;
                arguments
                    << XmlOptions::get().raw( "GBBOX" )
                    << "mkdir" << "-p"
                    << path.file();

                // run and emit signal
                _newProcess().start( arguments );
                emit commandStarted( command );
                return;

            }

        }

        case AlienCommand::Locate:
        {
            const FileInfo& source( command.argument(0) );
            if( source.isLocal() ) break;
            else {

                QString buffer;
                QTextStream( &buffer ) << "Requesting location of \"" << source.file() << "\"" << endl;
                emit logAvailable( buffer, FORMAT::Bold );
                emit messageAvailable( "Requesting file location" );

                QStringList arguments;
                arguments << XmlOptions::get().raw( "GBBOX" )
                    << "whereis"
                    << source.file();

                // run and emit signal
                _newProcess().start( arguments );
                emit commandStarted( command );
                return;

            }

        }

        case AlienCommand::Rename:
        {

            const FileInfo& source( command.argument(0) );
            const FileInfo& destination( command.argument(1) );
            QString buffer;
            QTextStream( &buffer ) << "Renaming \"" << source.file() << "\"" << endl;
            emit logAvailable( buffer, FORMAT::Bold );
            emit messageAvailable( source.isFolder() ? "Renaming folder":"Renaming file" );

            if( source.isLocal() )
            {

                // this probably fails on windows
                if( !source.file().rename( destination.file() ) )
                {

                    failedCommands_.append( command );
                    if( ( commands_.isEmpty() || commands_.back().type() != AlienCommand::FlushErrors ) )
                    { _addCommand( AlienCommand::FlushErrors ); }

                } else {

                    emit fileInfoRemoved( source );
                    emit fileInfoAdded( destination );

                }

                break;

            } else {

                QStringList arguments;
                arguments << XmlOptions::get().raw( "GBBOX" )
                    << "mv"
                    << source.file()
                    << destination.file();

                // run and emit signal
                _newProcess().start( arguments );
                emit commandStarted( command );
                return;

            }

        }

        case AlienCommand::Remove:
        {

            const FileInfo& path( command.argument(0) );
            QString buffer;
            QTextStream( &buffer ) << "Removing \"" << path.file() << "\"" << endl;
            emit logAvailable( buffer, FORMAT::Bold );
            emit messageAvailable( path.isFolder() ? "Removing folder":"Removing file" );

            if( path.isLocal() )
            {

                path.file().removeRecursive();
                if( path.file().exists() ) {

                    QString buffer;
                    QTextStream( &buffer ) << "Failed to remove file: " << path.file() << "." << endl;
                    emit logAvailable( buffer );

                    // store failed command
                    command.setErrorDetails( buffer );
                    failedCommands_.append( command );
                    if( ( commands_.isEmpty() || commands_.back().type() != AlienCommand::FlushErrors ) )
                    { _addCommand( AlienCommand::FlushErrors ); }

                } else emit fileInfoRemoved( path );

                break;

            } else {

                // build command
                QStringList arguments;
                arguments << XmlOptions::get().raw( "GBBOX" );

                if( path.isFolder() ) arguments << "rmdir" << path.file();
                else arguments << "rm" << path.file();

                // run and emit signal
                _newProcess().start( arguments );
                emit commandStarted( command );
                return;

            }

        }

        case AlienCommand::Copy:
        {
            FileInfo source( command.argument(0) );
            FileInfo destination( command.argument(1) );
            currentTransfered_ = 0;
            replica_ = 1;
            maxReplica_ = 1;

            QString buffer;
            QTextStream( &buffer )
                << "Copying \"" << source.file() << "\" to "
                << ( destination.isFolder() ? "folder":"file" )
                << " \"" << destination.file() << "\"" << endl;
            emit logAvailable( buffer, FORMAT::Bold );

            buffer.clear();
            QTextStream( &buffer ) << "Copying " << (source.isFolder() ? "folder":"file" ) << " " << source.file();
            emit messageAvailable( buffer );

            if( source.isLocal() && destination.isLocal() )
            {

                // setup thread and run
                thread_.setFile( source.file() );
                thread_.setDestination( destination.file() );
                thread_.setCommand( FileThread::Copy );
                thread_.start();

                emit commandStarted( command );
                return;

            } else {

                QStringList arguments;
                arguments << XmlOptions::get().raw( "ALIEN_CP" ) << "-n";
                arguments << QString( source.isLocal() ? "file:":"alien:" ) + source.file();
                arguments << QString( destination.isLocal() ? "file:":"alien:" ) + destination.file();

                // run and emit signal
                _newProcess().start( arguments );
                emit commandStarted( command );
                return;

            }

        }

        case AlienCommand::Open:
        {

            // update logs
            const FileInfo& path( command.argument(0) );
            const File application( command.argument(1).file() );
            QString buffer;
            QTextStream( &buffer ) << "Opening \"" << path.file() << "\"" << endl;
            emit logAvailable( buffer, FORMAT::Bold );
            emit messageAvailable( path.isFolder() ? "Opening folder":"Opening file" );

            // check file info type
            Q_ASSERT( path.isLocal() );
            Q_ASSERT( !application.isEmpty() );

            ( Command( application ) << path.file() ).run();
            break;

        }

        case AlienCommand::Find:
        {

            emit logAvailable( "Requesting file collection\n", FORMAT::Bold );
            emit messageAvailable( "Requesting file collection" );

            // run find command
            const FileInfo& path( command.argument(0) );
            if( path.isRemote() )
            {

                // get path and pattern
                QString pattern( command.argument(1).file() );
                int type( command.argument(2).file().toInt() );

                QStringList arguments;
                arguments
                    <<  XmlOptions::get().raw( "GBBOX" )
                    <<  "find";
                if( type == 1 ) arguments << "-x" << "collection";
                arguments << path.file() << pattern;

                // run and emit signal
                _newProcess().start( arguments );
                emit commandStarted( command );
                return;

            } else {

                // do nothing for local request
                break;

            }

        }

        default:
        break;
    }

    emit commandCompleted( command );
    commands_.removeFirst();

    // run next command/macro
    if( commands_.isEmpty() ) _macroCompleted();
    else _runCommand();

    return;

}

//________________________________________________________________________
void AlienInterface::_checkConnection( void )
{

    Debug::Throw( "AlienInterface::_checkConnection.\n" );

    // Warning [Mon Oct 29 09:39:48 2012] GCLIENT::Reconnect
    // Warning [Mon Oct 29 09:39:48 2012] GCLIENT::COMMAND command failed
    // QRegExp regExp1( "GCLIENT::Reconnect" );
    // QRegExp regExp2( "GCLIENT::COMMAND command failed" );
    QRegExp regExp1( "(Error)|(Warning)\\s+\\[.+\\]\\s+GCLIENT::Reconnect" );
    QRegExp regExp2( "(Error)|(Warning)\\s+\\[.+\\]\\s+GCLIENT::COMMAND command failed" );
    if( stderrBuffer_.text().indexOf( regExp1 ) >= 0 && stderrBuffer_.text().indexOf( regExp2 ) >= 0 )
    {

        if( connected_ )
        {
            emit connectionLost( stderrBuffer_.text() );
            emit connectionStateChanged( false );
        }

        connected_ = false;

    } else if( !( connected_ && wasConnected_ ) ) {

        emit connectionStateChanged( true );
        wasConnected_ = true;
        connected_ = true;

    }

}

//________________________________________________________________________
void AlienInterface::_processProxyInfo( void )
{
    Debug::Throw( "AlienInterface::_processProxyInfo.\n" );

    // get current command
    AlienCommand command( commands_.front() );
    Q_ASSERT( command.type() == AlienCommand::CheckProxy );

    // proxy validity
    bool valid( false );

    // time left   : 6h:52m:40s
    QRegExp regexp( "time\\s+left\\s+:\\s+(\\d+)h:(\\d+)m:(\\d+)s" );
    if( stderrBuffer_.text().indexOf( regexp ) >= 0 )
    {

        const int totalTime = 3600*regexp.cap(1).toInt() + 60*regexp.cap(2).toInt() + regexp.cap(3).toInt();
        valid = ( totalTime > 0 );

    }

    // if invalid emit signal
    if( !valid )
    {
        command.setErrorDetails( stderrBuffer_.text() );
        failedCommands_.append( command );
        if( ( commands_.isEmpty() || commands_.back().type() != AlienCommand::FlushErrors ) )
        { _addCommand( AlienCommand::FlushErrors ); }

    }

}

//________________________________________________________________________
void AlienInterface::_processHomeDirectory( bool alsoUpdate )
{

    Debug::Throw( "AlienInterface::_processHomeDirectory.\n" );
    QString defaultUser;

    // for some reason default user is written to stderr
    foreach( QString buffer, QStringList() << stdoutBuffer_.text() << stderrBuffer_.text() )
    {
        QTextStream stream( &buffer );

        // split into lines
        QStringList lines;
        QString line;
        while( !( line = stream.readLine() ).isNull() ) lines << line;

        // read default user
        if( !lines.isEmpty() && !_isError( line = lines.back() ) )
        { defaultUser = line.trimmed(); }

        if( !defaultUser.isEmpty() ) break;

    }

    // build path from user name
    if( defaultUser.isEmpty() )
    {
        emit messageAvailable( "Failed to retrieve default user" );
        return;
    }

    // store default user in options
    XmlOptions::get().setRaw( "ALIEN_USER", defaultUser );

    // build home path
    QString path;
    QTextStream( &path ) << "/alice/cern.ch/user/" << defaultUser[0] << "/" << defaultUser;


    emit messageAvailable( "Remote home directory available" );
    emit homeAvailable( FileInfo( path, FileInfo::Remote|FileInfo::Folder ), alsoUpdate );

}

//________________________________________________________________________
void AlienInterface::_processDirectoryContent( const FileInfo& path )
{
    Debug::Throw( "AlienInterface::_processDirectoryContent.\n" );

    // parsing regexp
    // prototype: drwxr-xr-x hupereir hupereir 0 Jun 25 17:26 .plugin_test_copy
    QRegExp regexp(
        "((?:d|r|w|x|-){10,10})\\s+"
        "(\\S+)\\s+(\\S+)\\s+"
        "(\\d+)\\s+(\\S{3,3}\\s\\d{2,2}\\s\\d{2,2}\\:\\d{2,2})\\s+"
        "(\\S+)"
        );

    // date prototype
    QRegExp dateRegexp( "(\\S{3,3})\\s(\\d{2,2})\\s(\\d{2,2})\\:(\\d{2,2})" );

    int position = 0;
    while( ( position = stdoutBuffer_.text().indexOf( regexp, position ) ) >= 0 )
    {

        FileInfo fileInfo( File( regexp.cap(6) ).addPath( path.file() ), FileInfo::Remote );

        // tag file as hidden
        if( fileInfo.file().localName().startsWith( '.' ) )
        { fileInfo.setIsHidden(); }

        if( _showHiddenFiles() || !fileInfo.file().isHidden() )
        {
            fileInfo.setUser( regexp.cap(2) );
            fileInfo.setGroup( regexp.cap(3) );
            fileInfo.setSize( regexp.cap(4).toUInt() );
            fileInfo.setPermissions( regexp.cap(1) );

            if( regexp.cap(5).indexOf( dateRegexp ) >= 0 )
            {
                TimeStamp lastModified( TimeStamp::now() );
                lastModified.setSeconds( 0 );
                lastModified.setMonth( dateRegexp.cap(1) );
                lastModified.setDay( dateRegexp.cap(2).toInt() );
                lastModified.setHours( dateRegexp.cap(3).toInt() );
                lastModified.setMinutes( dateRegexp.cap(4).toInt() );
                lastModified.makeTime();
                fileInfo.setLastModified( lastModified );
            }

            _addFile( fileInfo );
        }

        position += regexp.matchedLength();
    }
}


//________________________________________________________________________
void AlienInterface::_processLocation( const FileInfo& fileInfo )
{
    Debug::Throw( "AlienInterface::_processLocation.\n" );

    // SE => ALICE::FZK::SE  pfn =>root://alice-disk-se.gridka.de:1094//11/11031/f90a225e-1e9c-11e2-8b67-9fab084338ba
    QRegExp regexp( "SE\\s+=>\\s+(\\S+)\\s+pfn\\s+=>\\s*(\\S+)" );

    QStringList out;
    int position = 0;
    while( ( position = stdoutBuffer_.text().indexOf( regexp, position ) ) >= 0 )
    {
        const QString storageElement( regexp.cap(1) );
        const QString filename( regexp.cap(2) );
        out << storageElement;
        position += regexp.matchedLength();
    }

    if( !out.empty() ) emit locationAvailable( fileInfo, out );

}

//____________________________________________________________
void AlienInterface::_resetBuffers( void )
{
    Debug::Throw( "AlienInterface::_resetBuffer.\n" );
    stdoutBuffer_.clear();
    stderrBuffer_.clear();
    stdoutIndex_ = 0;
    stderrIndex_ = 0;
}

//___________________________________________
void AlienInterface::_resetProcess( void )
{
    if( process_ )
    {
        process_->deleteLater();
        process_ = 0;
    }
}

//____________________________________________________________
CustomProcess& AlienInterface::_newProcess( void )
{

    Debug::Throw( "AlienInterface::_newProcess.\n" );

    // reset old process
    _resetProcess();

    // create new process and assign customized environment
    process_ = new CustomProcess( this );
    process_->setProcessEnvironment( systemEnvironment_ );

    connect( &_process(), SIGNAL( error(  QProcess::ProcessError ) ), SLOT( _error( QProcess::ProcessError ) ) );
    connect( &_process(), SIGNAL( finished( int, QProcess::ExitStatus ) ), this, SLOT( _commandCompleted( int, QProcess::ExitStatus ) ) );
    connect( &_process(), SIGNAL( readyReadStandardOutput() ), SLOT( _readStdout() ) );
    connect( &_process(), SIGNAL( readyReadStandardError() ), SLOT( _readStderr() ) );
    return _process();

}

//____________________________________________________________
bool AlienInterface::_isError( const QString& value ) const
{
    if( value.indexOf( "verify failed" ) >= 0 ) return true;
    else if( value.indexOf( "command failed" ) >= 0 ) return true;
    else if( value.indexOf( "err -1" ) >= 0 ) return true;
    else if( value.indexOf( "err 10" ) >= 0 ) return true;
    else return false;
}

//_________________________________________________________________________________________________
void AlienInterface::_removePath( File& source, File path ) const
{
    if( !path.endsWith( '/' ) ) path += '/';
    if( !source.startsWith( path ) ) return;
    source = source.mid( path.size() );
}

//_________________________________________________________________
void AlienInterface::_resetTotalSize( void )
{
    Debug::Throw( "AlienInterface::_resetTotalSize.\n" );

    // compute total size to transfer
    total_ = 1;
    transfered_ = 0;
}

//_________________________________________________________________
bool AlienInterface::_updateTotalSize( void )
{

    Debug::Throw( "AlienInterface::_updateTotalSize.\n" );
    qint64 total(0);
    foreach( const FileInfo& fileInfo, files_ )
    { total += fileInfo.size(); }

    if( total >= 0 )
    {

        total_+= total;
        return true;

    } else return false;

}

//_________________________________________________________________
void AlienInterface::_computeTotalSize( void )
{
    Debug::Throw( "AlienInterface::_computeTotalSize.\n" );

    // compute total size to transfer
    total_ = 0;
    transfered_ = 0;
    foreach( const FileInfo& fileInfo, filesRecursive_ )
    { total_ += fileInfo.size(); }

    // needed because progress-bars start "oscillating"
    // when a total size of 1 is set.
    if( total_ <= 0 ) total_ = 1;

    return;
}

//_______________________________________
bool AlienInterface::_ignoreError( const AlienCommand& command, const QString& message ) const
{

    // check connection state
    if( !( connected_ || command.type() == AlienCommand::Connect ) ) return false;

    // check command type. Ignore "already exists" errors for folder creation
    switch( command.type() )
    {

        case AlienCommand::CreateDirectory:
        {
            // ignore errors for creating directories when destination already exists
            if( message.indexOf( "already exists." ) >= 0 ) return true;
            else break;
        }

        default:
        break;

    }

    return false;

}

//_______________________________________
void AlienInterface::_updateConfiguration( void )
{
    Debug::Throw( "AlienInterface::_updateConfiguration.\n" );

    // get system environment
    systemEnvironment_ = QProcessEnvironment::systemEnvironment();

    // get certificates and check
    const File certificates( XmlOptions::get().raw( "CERTIFICATES_PATH" ) );
    if( certificates.exists() && certificates.isDirectory() )
    {
        systemEnvironment_.insert( "XrdSecSSLCADIR", certificates );
        systemEnvironment_.insert( "X509_CERT_DIR", certificates );
    }

    // get gshell root path from gbbox binary
    const QString gbboxCommand( XmlOptions::get().raw( "GBBOX" ) );
    const QString match( "/bin/gbbox" );
    const int position( gbboxCommand.lastIndexOf( match ) );
    File gShellPath;
    if( position == gbboxCommand.length() - match.length() )
    { gShellPath = gbboxCommand.left( position ) + "/"; }

    // check
    if( gShellPath.exists() && gShellPath.isDirectory() )
    { systemEnvironment_.insert( "GSHELL_ROOT", gShellPath ); }

}
