#ifndef AlienInterface_h
#define AlienInterface_h

// $Id: AlienInterface.h,v 1.36 2012/11/30 10:28:00 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "Debug.h"
#include "AlienCommand.h"
#include "AlienMacro.h"
#include "FileInfo.h"
#include "FileThread.h"
#include "MessageBuffer.h"
#include "TextFormat.h"

#include <QtCore/QObject>
#include <QtCore/QProcess>
#include <QtCore/QProcessEnvironment>
#include <QtCore/QString>
#include <QtCore/QMap>

class CustomProcess;

//! Job managment
class AlienInterface:public QObject, public Counter
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor
    AlienInterface( QObject* parent = 0 );

    //! destructor
    virtual ~AlienInterface( void );

    //! find certificates
    void findCertificates( void );

    //! clear failed commands
    void clearFailedCommands( void )
    { failedCommands_.clear(); }

    signals:

    //! connection lost
    void connectionLost( const QString& = QString() );

    //! connection state changed
    void connectionStateChanged( bool );

    //! home directory available
    void homeAvailable( const FileInfo&, bool );

    //! directory content is available
    void contentAvailable( const FileInfo&, const FileInfo::List& );

    //! directory size
    void sizeAvailable( const FileInfo&, qint64 );

    //! collection available
    void collectionAvailable( const QString& );

    //! scratch file created
    void scratchFileCreated( const File& );

    //! file added
    void fileInfoAdded( const FileInfo& );

    //! file removed
    void fileInfoRemoved( const FileInfo& );

    //! location available
    void locationAvailable( const FileInfo&, const QStringList& );

    //!@name macro interface
    //@{

    //! macro added
    void macroAdded( const AlienMacro& );

    //! macros added
    void macrosAdded( const AlienMacro::List& );

    //! macro started
    void macroStarted( const AlienMacro& );

    //! macro completed
    void macroCompleted( const AlienMacro& );

    //! macro failed
    void macroFailed( const AlienMacro&, const QString& = QString() );

    //@}

    //!@name command interface
    //@{

    //! command added
    void commandAdded( const AlienCommand& );

    //! commands added
    void commandsAdded( const AlienCommand::List& );

    //! command started
    void commandStarted( const AlienCommand& );

    //! command completed
    void commandCompleted( const AlienCommand& );

    //! commands failed
    void commandsFailed( const AlienCommand::List& );

    //@}

    //! emitted when busy state is changed
    void busyStateChanged( bool );

    //! emmitted when busy processing commands
    void busy( void );

    //! emmitted when there is no more commands in pile
    void ready( void );

    //! logAvailable
    void logAvailable( const QString&, FORMAT::TextFormatFlags = FORMAT::Default );

    //! message
    void messageAvailable( const QString& );

    //! emitted to show transfer progress
    void maxProgressAvailable( int );

    //! emitted to show transfer progress
    void progressAvailable( int );

    public slots:

    //! add command
    void addMacro( const AlienMacro& );

    //! add commands
    void addMacros( const AlienMacro::List& );

    //! abort all macros
    void abortAll( void );

    //! abort current macro
    void abortCurrent( void );

    //! show hidden files
    bool setShowHiddenFiles( bool value )
    {
        if( showHiddenFiles_ == value ) return false;
        showHiddenFiles_ = value;
        return true;
    }

    protected slots:

    //!@name macro interface
    //@{

    //! run next macro in pile
    void _runMacro( void );

    //! take action when current macro completed
    void _macroCompleted( bool error = false );

    //@}

    //!@name command interface
    //@{

    //! take action when current command completed
    void _commandCompleted( int = 0, QProcess::ExitStatus = QProcess::NormalExit );

    //@}

    //! error from process
    void _error( QProcess::ProcessError );

    //! read on-fly from process
    void _readStdout( void );

    //! read on-fly from process
    void _readStderr( void );

    //! process files from thread
    void _processFiles( const File::List& );

    //! process size from thread
    void _processSize( qint64 );

    protected:

    //!@name command interface
    //@{

    //! add command
    void _addCommand( const AlienCommand& );

    //! add commands
    void _addCommands( const AlienCommand::List& );

    //! run next command in pile
    void _runCommand( void );

    //@}

    //! show hidden files
    bool _showHiddenFiles( void ) const
    { return showHiddenFiles_; }

    //! check connection
    void _checkConnection( void );

    //! process proxy
    void _processProxyInfo( void );

    //! process user list
    void _processHomeDirectory( bool = false );

    //! process directory content
    void _processDirectoryContent( const FileInfo& );

    //! process remote file location
    void _processLocation( const FileInfo& );

    //! reset buffers
    void _resetBuffers( void );

    //! reset process
    void _resetProcess( void );

    //! create new process to run command
    CustomProcess& _newProcess( void );

    //! process
    bool _hasProcess( void ) const
    { return process_; }

    //! process
    CustomProcess& _process( void ) const
    { return *process_; }

    //! attempt at identifying error lines
    bool _isError( const QString& ) const;

    //! add file to recursive list
    void _addFileRecursive( const FileInfo& file )
    { filesRecursive_ << file; }

    //! clear list of files
    void _clearFilesRecursive( void )
    { filesRecursive_.clear(); }

    //! add file
    void _addFile( const FileInfo& fileInfo )
    { files_ << fileInfo; }

    //! clear files
    void _clearFiles( void )
    { files_.clear(); }

    //! remove path from a given file
    void _removePath( File& source, File path ) const;

    //! reset total and transfered size
    void _resetTotalSize( void );

    //! update total size from file_ content
    bool _updateTotalSize( void );

    //! compute total size and reset transfered size
    void _computeTotalSize( void );

    //! update transfered size
    void _updateTransferedSize( const FileInfo& file )
    { transfered_ += file.size(); }

    //! true if given error is to be ignored
    bool _ignoreError( const AlienCommand&, const QString& ) const;

    private slots:

    //! update configuration
    void _updateConfiguration( void );

    private:

    //!@name stored url informations for remote listing
    //@{

    //! list of files
    /*!
    filesRecursive_ is used to store all files read either remotely
    or locally, that can be retrieved internally
    */
    FileInfo::List filesRecursive_;

    //! list of files
    /*!
    files_ store all files read from a single "ls" command,
    they are copied into filesRecursives_ when several such commands
    must be called in a row
    */
    FileInfo::List files_;

    //@}

    //! true if hidden files must be shown
    bool showHiddenFiles_;

    //! custom process
    CustomProcess* process_;

    //! file thread, to handle local file manipulations
    FileThread thread_;

    //! stdout buffer
    MessageBuffer stdoutBuffer_;

    //! stderr buffer
    MessageBuffer stderrBuffer_;

    //! stdout index
    int stdoutIndex_;

    //! stderr index
    int stderrIndex_;

    //! macros
    AlienMacro::List macros_;

    //! commands
    AlienCommand::List commands_;

    //! failed commands
    AlienCommand::List failedCommands_;

    //! system environment
    QProcessEnvironment systemEnvironment_;

    //! true if connection was valid at least once
    bool wasConnected_;

    //! connected flag
    bool connected_;

    //! total size to transfer
    qint64 total_;

    //! transfered size
    qint64 transfered_;

    //! transfered size
    qint64 currentTransfered_;

    //! replica
    int replica_;

    //! replica
    int maxReplica_;

};

#endif
