#ifndef AlienMacro_h
#define AlienMacro_h

// $Id: AlienMacro.h,v 1.12 2012/11/29 14:09:53 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "Counter.h"
#include "Debug.h"
#include "FileInfo.h"
#include "QOrderedSet.h"

#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QHash>
#include <QtCore/QList>

//! handles top commands
class AlienMacro: public Counter
{

    public:

    //! enumeration for command type
    enum Type
    {
        None,
        Add,
        Clear,
        FlushErrors,
        Connect,
        CheckProxy,
        HomeDirectory,
        CreateDirectory,
        ShowDirectory,
        Copy,
        Locate,
        Rename,
        Remove,
        Open,
        Find,
        ListRecursive,
        SizeRecursive,
        CopyRecursive,
        RemoveRecursive,
        ClearSelection,
        Select
    };

    //! flags
    enum Flag
    {
        NoFlag = 0,
        IgnoreErrors = 1<<0,
        IgnoreDisconnect = 1<<1
    };

    Q_DECLARE_FLAGS(Flags, Flag)


    //! list of commands
    typedef QList<AlienMacro> List;

    //! constructor
    AlienMacro( Type = None, Flags = NoFlag );

    //! destructor
    virtual ~AlienMacro( void )
    {}

    //! equal to operator
    virtual bool operator == ( const AlienMacro& other ) const
    { return id() == other.id(); }

    //! less than operator
    virtual bool operator < ( const AlienMacro& other ) const
    { return id() < other.id(); }

    //!@name accessors
    //@{

    //! command name
    QString name( void ) const
    { return name( type() ); }

    //! map Type to command name
    static QString name( const Type& type );

    //! id
    int id( void ) const
    { return id_; }

    //! type
    Type type( void ) const
    { return type_; }

    //! flags
    Flags flags( void ) const
    { return flags_; }

    //! flags
    bool hasFlag( Flag flag ) const
    { return flags_&flag; }

    // true if arguments are passed
    bool hasArguments( void ) const
    { return !arguments_.isEmpty(); }

    //! argument size
    int argumentCount( void ) const
    { return arguments_.size(); }

    //! true if argument is present
    bool hasArgument( int index ) const
    { return index < arguments_.size(); }

    //! get argument
    const FileInfo& argument( int index ) const
    { return arguments_[index]; }

    //! arguments
    const FileInfo::List& arguments( void ) const
    { return arguments_; }

    //@}

    //!@name modifiers
    //@{

    //! add argument
    AlienMacro& addArgument( const FileInfo& argument )
    {
        arguments_ << argument;
        return *this;
    }

    //! push arguments
    AlienMacro& operator << (const FileInfo& argument )
    { return addArgument( argument ); }

    //! clear arguments
    void clearArguments( void )
    { arguments_.clear(); }

    //@}

    private:

    //! static id counter
    static unsigned int& _counter( void );

    //! map types to command name
    typedef QHash<Type,QString> NameMap;

    //! name map
    static NameMap& _names( void );

    //! initialize command names
    static void _initializeNames( void );

    //! unique id
    int id_;

    //! type
    Type type_;

    //! flags
    Flags flags_;

    //! arguments
    FileInfo::List arguments_;

};

Q_DECLARE_OPERATORS_FOR_FLAGS( AlienMacro::Flags )

#endif
