// $Id: AlienMacroModel.cpp,v 1.5 2012/11/22 22:25:32 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "AlienMacroModel.h"

#include "CustomPixmap.h"
#include "Icons.h"
#include "IconEngine.h"
#include "IconSize.h"
#include "TimeStamp.h"
#include "XmlOptions.h"

#include <QIcon>

//_______________________________________________
const QString AlienMacroModel::columnTitles_[ AlienMacroModel::nColumns ] =
{ "Name", "File" };

//_______________________________________________________________
AlienMacroModel::AlienMacroModel( QObject* parent ):
    ListModel<AlienMacro>( parent ),
    Counter( "AlienMacroModel" )
{

    Debug::Throw( "AlienMacroModel::AlienMacroModel.\n" );
    updateConfiguration();

}

//__________________________________________________________________
QVariant AlienMacroModel::data( const QModelIndex& index, int role ) const
{

    // check index, role and column
    if( !index.isValid() ) return QVariant();

    // retrieve associated macro
    const AlienMacro& macro( get()[index.row()] );

    // return text associated to macro and column
    if( role == Qt::DisplayRole ) {

        switch( index.column() )
        {

            case NAME: return macro.name();
            case FILE: return macro.argumentCount() > 0  ? macro.argument(0).file():QVariant();
            default: return QVariant();
        }
    }

    // return icon associated to file
    if( role == Qt::DecorationRole && index.column() == NAME )
    { return _icon(macro.type()); }

    return QVariant();

}

//__________________________________________________________________
QVariant AlienMacroModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    if(
        orientation == Qt::Horizontal &&
        role == Qt::DisplayRole &&
        section >= 0 &&
        section < nColumns )
    { return columnTitles_[section]; }

    // return empty
    return QVariant();

}

//_________________________________________________________
void AlienMacroModel::updateConfiguration( void )
{

    Debug::Throw( "AlienMacroModel::updateConfiguration.\n" );

    // install pixmaps
    _icons().clear();
    _installIcons();

    reset();

}

//_____________________________________________________________________
void AlienMacroModel::_installIcons( void ) const
{

    Debug::Throw( "AlienMacroList::_installIcons" );

    if( !_icons().empty() ) return;

    // type icons
    typedef QHash< AlienMacro::Type, QString > IconNames;
    IconNames typeNames;
    typeNames[AlienMacro::Add] = ICONS::ADD;
    typeNames[AlienMacro::Clear] = ICONS::DELETE;
    typeNames[AlienMacro::Connect] = ICONS::CONNECT;
    typeNames[AlienMacro::CheckProxy] = ICONS::CONNECT;
    typeNames[AlienMacro::HomeDirectory] = ICONS::HOME;
    typeNames[AlienMacro::HomeDirectory] = ICONS::HOME;
    typeNames[AlienMacro::CreateDirectory] = ICONS::CREATE_DIRECTORY;
    typeNames[AlienMacro::ShowDirectory] = ICONS::FIND;
    typeNames[AlienMacro::Copy] = ICONS::PASTE;
    typeNames[AlienMacro::Rename] = ICONS::RENAME;
    typeNames[AlienMacro::Remove] = ICONS::DELETE;
    typeNames[AlienMacro::Open] = ICONS::EDIT;
    typeNames[AlienMacro::Find] = ICONS::FIND;
    typeNames[AlienMacro::ListRecursive] = ICONS::FIND;
    typeNames[AlienMacro::SizeRecursive] = ICONS::FIND;
    typeNames[AlienMacro::CopyRecursive] = ICONS::PASTE;
    typeNames[AlienMacro::RemoveRecursive] = ICONS::PASTE;

    // load link pixmap
    for( IconNames::iterator iter = typeNames.begin(); iter != typeNames.end(); ++iter )
    { _icons().insert( iter.key(), IconEngine::get( iter.value() ) ); }

    return;
}

//_______________________________________________
AlienMacroModel::IconCache& AlienMacroModel::_icons( void )
{
    static IconCache cache;
    return cache;
}

//____________________________________________________________
const QIcon& AlienMacroModel::_icon( AlienMacro::Type type )
{

    IconCache::iterator iter = ( _icons().find( type ) );
    if( iter != _icons().end() ) return iter.value();
    else return _icons().insert( type, CustomPixmap().empty( IconSize( IconSize::Medium ) ) ).value();

}

//________________________________________________________
bool AlienMacroModel::SortFTor::operator () ( AlienMacro first, AlienMacro second ) const
{

    if( order_ == Qt::DescendingOrder ) std::swap( first, second );
    switch( type_ )
    {

        case AlienMacroModel::NAME:
        if( first.type() != second.type() ) return first.type() < second.type();
        else break;

        case AlienMacroModel::FILE:
        {
            const FileInfo source( first.argumentCount() > 0 ? first.argument(0).file():FileInfo() );
            const FileInfo destination( second.argumentCount() > 0 ? second.argument(0).file():FileInfo() );
            if( source.file() != destination.file() ) return source.file() < destination.file();
            else break;
        }

        default: break;
    }

    return first.id() < second.id();

}
