// $Id: FileInformationDialog.cpp,v 1.15 2012/10/28 20:46:32 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "FileInformationDialog.h"

//_________________________________________________________
FileInformationDialog::FileInformationDialog( QWidget* parent, const FileInfo& fileInfo ):
    BaseFileInformationDialog( parent ),
    fileInfo_( fileInfo ),
    busy_( false )
{
    Debug::Throw( "FileInformationDialog::FileInformationDialog.\n" );

    // file name
    const File& file( fileInfo.file() );
    setWindowTitle( file.localName() + " - Gridftp" );

    setFile( file.localName() );
    setPath( file.path() );

    // type
    setType( fileInfo.typeString() );

    // size
    if( fileInfo.isDocument() && !fileInfo.isLink() )
    { setSize( fileInfo.size() ); }

    // time stamps
    if( fileInfo.isLocal() )
    {
        setCreated( file.created() );
        setAccessed( file.lastAccessed() );
    }

    setModified( fileInfo.lastModified() );

    // add location
    locationId_ = addRow( "Remote sites:" );

    // add full size button
    if( fileInfo.isFolder() )
    {

        QHBoxLayout* hLayout;
        pageLayout().addLayout( hLayout = new QHBoxLayout() );
        hLayout->addStretch();
        hLayout->addWidget( sizeRequestButton_ = new QPushButton( "Compute Total Size", &mainPage() ) );
        connect( sizeRequestButton_, SIGNAL( clicked( void ) ), SLOT( _requestSize( void ) ) );
        sizeRequestButton_->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed );
        sizeRequestButton_->setFixedSize( sizeRequestButton_->sizeHint() );
    }

    setPermissions( fileInfo.permissions() );
    setUser( fileInfo.user() );
    setGroup( fileInfo.group() );

    adjustSize();

}

//_________________________________________________________
void FileInformationDialog::macroStarted( const AlienMacro& macro )
{

    // check macro
    if( !( macro.type() == AlienMacro::SizeRecursive && macro.argument(0) == fileInfo_ ) ) return;

    // update
    busy_ = true;
    sizeRequestButton_->setText( "Abort" );
    sizeRequestButton_->setEnabled( true );
}

//_________________________________________________________
void FileInformationDialog::macroCompleted( const AlienMacro& macro )
{

    // check macro
    if( !( macro.type() == AlienMacro::SizeRecursive && macro.argument(0) == fileInfo_ ) ) return;

    // update
    busy_ = false;
    sizeRequestButton_->setText( "Compute Total Size" );
    sizeRequestButton_->setEnabled( true );

}

//_________________________________________________________
void FileInformationDialog::updateSize( const FileInfo& fileInfo, qint64 size )
{ if( fileInfo == fileInfo_ ) setSize( size ); }

//_________________________________________________________
void FileInformationDialog::requestLocation( void )
{
    // check file info and request macro
    if( !( fileInfo_.isDocument() && fileInfo_.isRemote() ) ) return;
    emit macroRequested( AlienMacro( AlienMacro::Locate ) << fileInfo_ );
}

//_________________________________________________________
void FileInformationDialog::updateLocation( const FileInfo& fileInfo, const QStringList& locations )
{
    if( !( fileInfo == fileInfo_ ) ) return;

    if( locations.empty() ) setCustomValue( locationId_, QString() );
    else {

        QString buffer;
        QTextStream what( &buffer );
        bool first( true );
        foreach( const QString& location, locations )
        {
            if( first ) first = false;
            else what << endl;
            what << location;
        }

        setCustomValue( locationId_, buffer );

    }

}

//_________________________________________________________
void FileInformationDialog::_requestSize( void )
{
    Debug::Throw() << "FileInformationDialog::_requestSize." << endl;

    if( busy_ )
    {
        sizeRequestButton_->setText( "Compute Total Size" );
        busy_ = false;
        emit requestAbort();

    } else {

        AlienMacro::List macros;
        macros
            << AlienMacro::Clear
            << ( AlienMacro( AlienMacro::SizeRecursive ) << fileInfo_ );
        sizeRequestButton_->setEnabled( false );
        emit macrosRequested( macros );

    }
}

//_________________________________________________________
void FileInformationDialog::closeEvent( QCloseEvent* event )
{
    Debug::Throw( "FileInformationDialog::closeEvent.\n" );

    // abort command
    if( busy_ ) emit requestAbort();
    CustomDialog::closeEvent( event );

}
