
// $Id: FileSelectionDialog.cpp,v 1.4 2012/10/22 11:40:47 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY;  without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA   02111-1307 USA
*
*
*******************************************************************************/

#include "Debug.h"
#include "FileSelectionDialog.h"
#include "TreeView.h"
#include "XmlOptions.h"

#include <QtGui/QLabel>

//______________________________________________________________________
FileSelectionDialog::FileSelectionDialog( QWidget* parent, const QString& label ):
    QuestionDialog( parent, label ),
    model_( this, FileInfo::None )
{

    Debug::Throw( "FileSelectionDialog::FileSelectionDialog.\n" );
    setSizeGripEnabled( true );

    mainWidget_ = new QWidget( this );

    // main (working) layout
    mainLayout_ = new QVBoxLayout();
    mainLayout_->setSpacing( 10 );
    mainLayout_->setMargin( 0 );
    mainWidget_->setLayout( mainLayout_ );

    // horizontal layout for list and comments
    QHBoxLayout* hLayout( new QHBoxLayout() );
    hLayout->setSpacing( 5 );
    hLayout->setMargin( 0 );
    mainLayout_->addLayout( hLayout, 1 );

    // create list
    hLayout->addWidget( list_ = new TreeView( mainWidget_ ) );
    _list().setSelectionMode( QAbstractItemView::ContiguousSelection );
    _list().setModel( &model_ );
    _list().setMask( XmlOptions::get().get<unsigned int>( "LOCAL_LIST_MASK" ) );

    // work layout
    QVBoxLayout *vLayout = new QVBoxLayout();
    vLayout->setSpacing(0);
    vLayout->setMargin(0);
    hLayout->addLayout( vLayout );

    work_ = new QVBoxLayout();
    work_->setSpacing( 5 );
    work_->setMargin( 0 );
    vLayout->addLayout( work_ );
    vLayout->addStretch(1);

    // select all
    QPushButton* button;
    _workLayout().addWidget( button = new QPushButton( "&Select All", mainWidget_ ) );
    button->setToolTip( "Select all files in list" );
    connect( button, SIGNAL( clicked() ), list_, SLOT( selectAll() ) );

    // deselect all
    _workLayout().addWidget( clearSelectionButton_ = new QPushButton( "&Clear Selection", mainWidget_ ) );
    clearSelectionButton_->setToolTip( "Deselect all files in list" );
    connect( clearSelectionButton_, SIGNAL( clicked() ), list_, SLOT( clearSelection() ) );

    // details
    mainWidget_->adjustSize();
    setDetails( mainWidget_ );
    showDetails();

    // list connection
    connect( _list().selectionModel(), SIGNAL( selectionChanged(const QItemSelection &, const QItemSelection &) ), SLOT( _updateButtons() ) );
    _updateButtons();

}

//____________________________________________________________________
void FileSelectionDialog::setFiles( const File& localPath, const File& remotePath, const FileInfo::List& files )
{

    Debug::Throw( "FileSelectionDialog::setFiles.\n" );

    // store path
    localPath_ = localPath;
    remotePath_ = remotePath;

    // add all conflicting items to list
    FileInfo::List local;
    foreach( const FileInfo& fileInfo, files )
    {

        // some debugging
        if( !fileInfo.isConflict() ) Debug::Throw() << "FileSelectionDialog::setFiles - file: " << fileInfo.file() << " no conflict" << endl;
        else Debug::Throw() <<  "FileSelectionDialog::setFiles - file: " << fileInfo.file() << ( fileInfo.isOverwrite() ? " overwrite":" skip" ) << endl;

        FileInfo copy( fileInfo );
        copy.setFile( fileInfo.file().addPath( fileInfo.isLocal() ? localPath_:remotePath_ ) );
        local << copy;

    }

    model_.set( local );
    _list().resizeColumnToContents( FileInfoModel::ACTION );
    _list().resizeColumnToContents( FileInfoModel::FILE );
    _list().selectAll();

}

//____________________________________________________________________
void FileSelectionDialog::updateFiles( FileInfo::List& files )
{

    Debug::Throw( "FileSelectionDialog::updateFiles.\n" );

    // retrieve all items and put corresponding file info into output list
    const FileInfo::List& local( model_.get() );

    // should loop over files in input list
    for( FileInfo::List::iterator iter = files.begin(); iter != files.end(); ++iter )
    {

        // look for matching FileInfo in input list
        FileInfo fileInfo( *iter );
        fileInfo.setFile( fileInfo.file().addPath( fileInfo.isLocal() ? localPath_:remotePath_ ) );

        // this will not work because the path has been modified
        FileInfo::List::const_iterator fileIter = std::find_if( local.begin(), local.end(), FileInfo::SameFileFTor( fileInfo.file() ) );
        if( fileIter == local.end() ) continue;

        // update action
        iter->setStatus( FileInfo::Overwrite, fileIter->isOverwrite() );

    }

    // some debugging output
    if( Debug::level() >= 1 )
    {
        Debug::Throw() << endl;
        foreach( const FileInfo& fileInfo, files )
        {
            if( !fileInfo.isConflict() ) Debug::Throw() << "FileSelectionDialog::updateFiles - file: " << fileInfo.file() << " no conflict" << endl;
            else Debug::Throw() <<  "FileSelectionDialog::updateFiles - file: " << fileInfo.file() << ( fileInfo.isOverwrite() ? " overwrite":" skip" ) << endl;
        }
    }

    return;

}

//____________________________________________________________________
void FileSelectionDialog::_updateButtons( void )
{
    Debug::Throw( "FileSelectionDialog::_updateButtons.\n" );
    bool hasSelection( !_list().selectionModel()->selectedRows().empty() );
    clearSelectionButton_->setEnabled( hasSelection );
}

