#ifndef FileSelectionFrame_h
#define FileSelectionFrame_h

// $Id: FileSelectionFrame.h,v 1.44 2012/12/07 18:14:17 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "AlienMacro.h"
#include "FileSelectionClipboard.h"
#include "FileInfoModel.h"
#include "XmlOptions.h"

#include <QtGui/QAbstractItemView>
#include <QtGui/QItemSelectionModel>
#include <QtGui/QMenu>

class PathEditor;
class BaseFileInfoToolTipWidget;

//! document/folder navigation
class FileSelectionFrame: public QWidget, public Counter
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor
    FileSelectionFrame( QWidget* parent, FileInfo::TypeFlags type );

    //! base directory
    FileInfo path( void ) const;

    //! type
    FileInfo::TypeFlags type( void ) const
    { return type_; }

    //! selected files
    FileInfo::List selection( void ) const;

    //! dirty state
    bool isDirty( void ) const
    { return dirty_; }

    //! dirty state
    void setDirty( bool value )
    { dirty_ = value; }

    //! returns true if this widget owns argument
    bool owns( QWidget* ) const;

    //! active state
    bool isActive( void ) const
    { return active_; }

    //! active state
    bool setActive( bool value )
    {
        if( active_ == value ) return false;
        active_ = value;
        _updateBackgroundColors();
        return true;
    }

    //! model
    const FileInfoModel& model( void ) const
    { return model_; }

    //! model
    FileInfoModel& model( void )
    { return model_; }

    //!@name mode
    //@{

    enum ViewMode
    {
        LIST,
        ICONS
    };

    //! mode
    bool setViewMode( ViewMode );

    //! mode
    ViewMode viewMode( void ) const
    { return viewMode_; }

    //@}

    //!@name actions
    //@{

    //! toggle visibility action
    QAction& visibilityAction( void ) const
    { return *visibilityAction_; }

    //! previous directory
    QAction& reloadAction( void ) const
    { return *reloadAction_; }

    //! previous directory
    QAction& previousDirectoryAction( void ) const
    { return *previousDirectoryAction_; }

    //! next directory
    QAction& nextDirectoryAction( void ) const
    { return *nextDirectoryAction_; }

    //! directory above
    QAction& parentDirectoryAction( void ) const
    { return *parentDirectoryAction_; }

    //! new folder action
    QAction& newFolderAction( void ) const
    { return *newFolderAction_; }

    //! cut selection action
    QAction& cutAction( void ) const
    { return *cutAction_; }

    //! copy selection action
    QAction& copyAction( void ) const
    { return *copyAction_; }

    //! paste clipboard action
    QAction& pasteAction( void ) const
    { return *pasteAction_; }

    //! paste clipboard action
    QAction& pasteInFolderAction( void ) const
    { return *pasteInFolderAction_; }

    //! rename selection action
    QAction& renameAction( void ) const
    { return *renameAction_; }

    //! remove selection action
    QAction& removeAction( void ) const
    { return *removeAction_; }

    //! open selection action
    QAction& openAction( void ) const
    { return *openAction_; }

    //! file properties
    QAction& filePropertiesAction( void ) const
    { return *filePropertiesAction_; }

    //! find collection
    QAction& findCollectionAction( void ) const
    { return *findCollectionAction_; }

    //@}

    signals:

    //! selection chaged
    void selectionChanged( void );

    //! request for opening a file
    void fileSelected( const FileInfo& );

    //! request for opening multiple files
    void openFilesRequested( const FileInfo::List& );

    //! clipboard changed
    void clipboardChanged( const FileSelectionClipboard& );

    //! emitted when actions enable state has changed
    void actionsChanged( void );

    //! request command
    void macroRequested( const AlienMacro& );

    //! request commands
    void macrosRequested( const AlienMacro::List& );

    //! request paste from clipboard
    void pasteClipboardRequested( const FileInfo& );

    //! request file information
    void requestFileInformation( const FileInfo&, const QIcon& );

    //! request collection
    void collectionRequested( const FileInfo& );

    public slots:

    //! home path
    void setHomePath( const File& );

    //! root path list
    void setRootPathList( const File::List& );

    //! current path
    void setPath( const File& );

    //! hidden files
    void setShowHiddenFiles( bool value )
    {
        if( showHiddenFiles_ == value ) return;
        showHiddenFiles_ = value;
        _reload();
    }

    //! navigator
    void setShowNavigator( bool value )
    {
        const FileInfo navigator( File(".."), FileInfo::TypeFlags( FileInfo::Folder|FileInfo::Navigator|type_ ) );
        if( value ) model_.add( navigator );
        else model_.remove( navigator );
    }

    //! clear selection
    void clearSelection( void );

    //! select item
    void select( const FileInfo& );

    protected slots:

    //! index activated
    void _indexActivated( const QModelIndex& );

    //! reload current directory
    void _reload( void );

    //! new folder
    void _newFolder( void );

    //! cut
    void _cut( void );

    //! copy
    void _copy( void );

    //! paste
    void _paste( void );

    //! paste
    void _pasteInFolder( void );

    //! rename
    void _rename( void );

    //! remove
    void _remove( void );

    //! open
    void _open( void );

    //! update context menu
    void _updateMenu( void );

    //! update actions, based on selection
    void _updateActions( void );

    //! navigation actions
    void _updateNavigationActions( void );

    //! show tooltip
    void _showToolTip( const QModelIndex& );

    //! toggle visibility
    void _toggleVisibility( bool value )
    {
        setVisible( value );
        XmlOptions::get().set<bool>( visibilityOptionName_, value );
    }

    //! file properties
    void _fileProperties( void );

    //! request collection
    void _requestCollection( void );

    protected:

    //! setup list
    void _setupList( void );

    //! show hidden files
    const bool& _showHiddenFiles( void ) const
    { return showHiddenFiles_; }

    //! update background colors
    void _updateBackgroundColors( void ) const;

    //! true if local
    bool _isLocal( void ) const
    { return !(type_ & FileInfo::Remote); }

    //! true if remote
    bool _isRemote( void ) const
    { return type_ & FileInfo::Remote; }

    private:

    //! actions
    void _installActions( void );

    //! path editor
    PathEditor* pathEditor_;

    //! tooltip widget
    BaseFileInfoToolTipWidget* toolTipWidget_;

    //! model
    FileInfoModel model_;

    //! selection model
    QItemSelectionModel selectionModel_;

    //! list of files
    QAbstractItemView* list_;

    //!@name menus
    //@{

    //! context menu
    QMenu* contextMenu_;

    //! column selection
    QMenu* columnSelectionMenu_;

    //! column sorting
    QMenu* columnSortingMenu_;

    //@}

    //!@name actions
    //@{

    //! toggle visibility
    QAction* visibilityAction_;

    //! reload
    QAction* reloadAction_;

    //! previous directory action
    QAction* previousDirectoryAction_;

    //! next directory action
    QAction* nextDirectoryAction_;

    //! parent directory action
    QAction* parentDirectoryAction_;

    //! new folder action
    QAction* newFolderAction_;

    //! cut selection action
    QAction* cutAction_;

    //! copy selection action
    QAction* copyAction_;

    //! paste
    QAction* pasteAction_;

    //! paste in folder action
    QAction* pasteInFolderAction_;

    //! rename action
    QAction* renameAction_;

    //! remove selection
    QAction* removeAction_;

    //! edit selection
    QAction* openAction_;

    //! file properties
    QAction* filePropertiesAction_;

    //! file collection
    QAction* findCollectionAction_;

    //@}

    //! base type (bitwise or of FileInfo::Type
    /*! this is used to distinguish between local and remote list views */
    FileInfo::TypeFlags type_;

    //! visibility option name
    QString visibilityOptionName_;

    //! view mode option name
    QString viewModeOptionName_;

    //! mode
    ViewMode viewMode_;

    //! true if dirty (path changed without directory update
    bool dirty_;

    //! show/hide hidden files
    bool showHiddenFiles_;

    //! active state
    bool active_;

};

#endif
