// $Id: FindCollectionDialog.cpp,v 1.8 2012/10/11 15:18:01 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "FindCollectionDialog.h"

#include "AnimatedLineEditor.h"
#include "AnimatedTextEditor.h"
#include "CustomComboBox.h"
#include "FileInfo.h"
#include "FileDialog.h"
#include "IconEngine.h"
#include "Icons.h"
#include "InformationDialog.h"
#include "XmlOptions.h"

#include <QtGui/QGridLayout>
#include <QtGui/QLabel>

//_________________________________________________
FindCollectionDialog::FindCollectionDialog( QWidget* parent ):
    CustomDialog( parent, CloseButton )
{

    Debug::Throw( "FindCollectionDialog::FindCollectionDialog.\n" );
    setOptionName( "FIND_COLLECTION_DIALOG" );

    // grid layout
    QGridLayout* gridLayout( new QGridLayout() );
    gridLayout->setSpacing(5);
    gridLayout->setMargin(0);
    mainLayout().addLayout( gridLayout );

    QLabel* label;

    // pattern
    gridLayout->addWidget( label = new QLabel( "File pattern:", this ), 1, 0, 1, 1 );
    gridLayout->addWidget( patternComboBox_ = new CustomComboBox( this ), 1, 1, 1, 2 );
    label->setAlignment( Qt::AlignVCenter|Qt::AlignRight );
    label->setBuddy( patternComboBox_ );
    patternComboBox_->setEditable( true );

    // retrieve applications from options
    foreach( const Option& application, XmlOptions::get().specialOptions( "COLLECTION_SEARCH_STRING" ) )
    { patternComboBox_->addItem( application.raw() ); }

    // type
    gridLayout->addWidget( label = new QLabel( "Output format:", this ), 2, 0, 1, 1 );
    gridLayout->addWidget( comboBox_ = new QComboBox( this ), 2, 1, 1,  1 );
    label->setAlignment( Qt::AlignVCenter|Qt::AlignRight );
    label->setBuddy( comboBox_ );
    comboBox_->setEditable( false );
    comboBox_->addItems( QStringList() << "Flat" << "Xml" );

    if( XmlOptions::get().contains( "DEFAULT_FIND_MODE" ) )
    { comboBox_->setCurrentIndex( XmlOptions::get().get<int>( "DEFAULT_FIND_MODE" ) ); }

    gridLayout->setColumnStretch( 2, 1 );

    // editor
    mainLayout().addWidget( editor_ = new AnimatedTextEditor( this ) );
    editor_->setReadOnly( true );

    // save button
    saveButton_ = new QPushButton( IconEngine::get( ICONS::SAVE_AS ), "Save As...", this );
    connect( saveButton_, SIGNAL( clicked() ), SLOT( _save() ) );
    buttonLayout().insertWidget(1, saveButton_ );

    // find button
    findButton_ = new QPushButton( IconEngine::get( ICONS::FIND ), "Find", this );
    connect( findButton_, SIGNAL( clicked() ), SLOT( _requestUpdate() ) );
    buttonLayout().insertWidget(1, findButton_ );

    // status label
    buttonLayout().insertWidget( 0, statusLabel_ = new QLabel( this ) );

    // connections
    connect( patternComboBox_, SIGNAL( textChanged( const QString& ) ), SLOT( _updateButtons( void ) ) );
    connect( comboBox_, SIGNAL( currentIndexChanged( const QString& ) ), SLOT( _updateButtons( void ) ) );
    connect( editor_, SIGNAL( textChanged( void ) ), SLOT( _updateButtons( void ) ) );

    patternComboBox_->setFocus();
    okButton().setDefault( false );
    okButton().setAutoDefault( false );

}

//_________________________________________________
void FindCollectionDialog::processCollection( const QString& constValue )
{

    Debug::Throw( "FindCollectionDialog::processCollection.\n" );
    QString value( constValue );

    // remove useless leading line (alien bug)
    QRegExp regExp( "AND THE OPTIONS ARE\\s*" );
    int position( value.indexOf( regExp ) );
    if( position >= 0 ) value = value.mid( position+regExp.matchedLength() );

    // remove trailing spaces at end of string (alien bug)
    if( comboBox_->currentIndex() == 1 ) value.replace( QRegExp( "\\s+$" ), "" );

    // assign text to editor
    editor_->setText( value );

    // count number of entries to assign to label
    int fileCount( 0 );
    if( comboBox_->currentIndex() == 0 )
    {

        QRegExp regExp( "(\\d+) files found" );
        if( value.indexOf( regExp ) >= 0 )
        { fileCount = regExp.cap(1).toInt(); }

    } else {

        // count number of occurences of <event name="XXX"> in string
        fileCount = value.count( QRegExp( "<event name=\"\\d+\">" ) );

    }

    if( fileCount == 0 ) statusLabel_->setText( "No match found" );
    else if( fileCount == 1 ) statusLabel_->setText( "Found one matching file" );
    else
    {
        QString buffer;
        QTextStream( &buffer ) << "Found " << fileCount << " matching files";
        statusLabel_->setText( buffer );
    }

}

//_________________________________________________
void FindCollectionDialog::_requestUpdate( void )
{

    Debug::Throw( "FindCollectionDialog::_requestUpdate.\n" );
    const QString pattern( patternComboBox_->currentText() );
    if( path_.isEmpty() || pattern.isEmpty() ) return;
    emit macroRequested( AlienMacro( AlienMacro::Find )
        << FileInfo( path_, FileInfo::Remote|FileInfo::Folder )
        << FileInfo( pattern )
        << FileInfo( QString().setNum( comboBox_->currentIndex() ) ) );

    // save options
    XmlOptions::get().set<int>( "DEFAULT_FIND_MODE", comboBox_->currentIndex() );
    XmlOptions::get().add( "COLLECTION_SEARCH_STRING", Option( pattern, Option::Recordable|Option::Current ) );

    // disable button
    findButton_->setEnabled( false );

}

//_________________________________________________
void FindCollectionDialog::_save( void )
{
    Debug::Throw( "FindCollectionDialog::_save.\n" );
    const QString text( editor_->toPlainText() );
    if( text.isEmpty() ) return;

    // get file
    FileDialog dialog( this );
    dialog.setFileMode( QFileDialog::AnyFile );
    dialog.setAcceptMode( QFileDialog::AcceptSave );
    if( !filename_.isEmpty() ) dialog.selectFile( filename_ );

    File file( dialog.getFile() );
    if( file.isNull() ) return;
    else file = file.expand();

    // check if file is directory
    if( file.isDirectory() )
    {
        QString buffer;
        QTextStream( &buffer ) << "File \"" << file << "\" is a directory. <Save> canceled.";
        InformationDialog( this, buffer ).exec();
        return;
    }

    // open output file
    QFile out( file );
    if( !out.open( QIODevice::WriteOnly ) )
    {
        QString buffer;
        QTextStream( &buffer ) << "Cannot write to file \"" << file << "\". <Save> canceled.";
        InformationDialog( this, buffer ).exec();
        return;
    }

    out.write( text.toAscii() );
    if( !text.isEmpty() && text[text.size()-1] != '\n' ) out.write( "\n" );

    // close
    out.close();

    // reassign filename
    filename_ = file;

}


//_________________________________________________
void FindCollectionDialog::_updateButtons( void )
{
    saveButton_->setEnabled( !editor_->toPlainText().isEmpty() );
    findButton_->setEnabled( !(path_.isEmpty() || patternComboBox_->currentText().isEmpty() ) );
    statusLabel_->clear();
}
