#ifndef MainWindow_h
#define MainWindow_h

// $Id: MainWindow.h,v 1.45 2012/12/12 09:52:23 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "AlienCommand.h"
#include "AlienMacro.h"
#include "BusyWidget.h"
#include "Counter.h"
#include "BaseMainWindow.h"
#include "Debug.h"
#include "FileInfo.h"
#include "FileInfoModel.h"
#include "FileSelectionClipboard.h"
#include "FileSelectionFrame.h"
#include "FileSystemWatcher.h"

class DockWidget;
class AlienCommandDialog;
class AlienInterface;
class PlacesWidget;
class FileIconProvider;
class LogDialog;
class ProgressStatusBar;

//! application main window
class MainWindow: public BaseMainWindow, public Counter
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor;
    MainWindow( QWidget* parent = 0 );

    //! initialize
    void initialize( const File& = File() );

    //! local selection frame
    FileSelectionFrame& localSelectionFrame( void ) const
    { return *localSelectionFrame_; }

    //! remote selection frame
    FileSelectionFrame& remoteSelectionFrame( void ) const
    { return *remoteSelectionFrame_; }

    //! selection frame matching a given type
    FileSelectionFrame& selectionFrame( unsigned int type ) const
    { return *((type & FileInfo::Remote) ? remoteSelectionFrame_:localSelectionFrame_); }

    //! list of frames
    QList<FileSelectionFrame*> selectionFrames( void ) const
    { return QList<FileSelectionFrame*>() << localSelectionFrame_ << remoteSelectionFrame_; }

    //! places
    DockWidget& placesPanel( void ) const
    { return *placesPanel_; }

    //! status bar
    ProgressStatusBar& statusBar( void )
    { return *statusbar_; }

    //! active frame
    FileSelectionFrame& activeFrame( void ) const;

    //!@name actions
    //@{

    // connect
    QAction& connectAction( void ) const
    { return *connectAction_; }

    // home directory
    QAction& homeDirectoryAction( void ) const
    { return *homeDirectoryAction_; }

    //! previous directory
    QAction& previousDirectoryAction( void ) const
    { return *previousDirectoryAction_; }

    //! next directory
    QAction& nextDirectoryAction( void ) const
    { return *nextDirectoryAction_; }

    // directory above
    QAction& parentDirectoryAction( void ) const
    { return *parentDirectoryAction_; }

    //! reload
    QAction& reloadAction( void ) const
    { return *reloadAction_; }

    //! abort
    QAction& abortAction( void ) const
    { return *abortAction_; }

    //! show hidden files action
    QAction& showHiddenFilesAction( void ) const
    { return *showHiddenFilesAction_; }

    //! command list action
    QAction& commandDialogAction( void ) const
    { return *commandDialogAction_; }

    //! log action
    QAction& logAction( void ) const
    { return *logAction_; }

    //! log action
    QAction& toggleLogAction( void ) const
    { return *toggleLogAction_; }

    //@}

    signals:

    //! request command
    void macroRequested( const AlienMacro& );

    //! request commands
    void macrosRequested( const AlienMacro::List& );

    //! scratch file created
    void scratchFileCreated( const File& );

    public slots:

    //! connection state changed
    void connectionStateChanged( bool );

    //! update home
    void updateHome( const FileInfo&, bool );

    //! update directory content
    void updateContent( const FileInfo&, const FileInfo::List& );

    //! add file info to relevant model
    void addFileInfo( const FileInfo& );

    //! remove file info from relevant model
    void removeFileInfo( const FileInfo& );

    //! macro started
    void macroStarted( const AlienMacro& );

    //! macro completed
    void macroCompleted( const AlienMacro& );

    //! connection lost
    void lostConnection( const QString& );

    //! commands failed
    void commandsFailed( const AlienCommand::List& );

    //! busy
    virtual void busy( void )
    { busyWidget_->start(); }

    //! busy
    virtual void idle( void )
    { busyWidget_->stop(); }

    protected:

    //! drag enter
    virtual void dragEnterEvent( QDragEnterEvent* );

    //! drag leave
    virtual void dragLeaveEvent( QDragLeaveEvent* );

    //! drag move
    virtual void dragMoveEvent( QDragMoveEvent* );

    //! close window event handler
    virtual void closeEvent( QCloseEvent* );

    //! check that transfer is possible
    bool _checkTransfer( const FileInfo&, const FileInfo::List& );

    //! paste selected files to destination
    void _paste( const FileInfo&, const FileInfo::List& );

    //! move selected files to destination
    /*! return updated clipboard */
    FileSelectionClipboard _move( const FileInfo&, const FileInfo::List& );

    //! get list of destination files by replacing source path by destination
    FileInfo::List _destinationList( const FileInfo&, const FileInfo::List&) const;

    //! update path
    void _updatePath( const FileInfo& );

    //! update window title
    void _updateWindowTitle( void );

    protected slots:

    //! update visibility actions
    void _updateVisibilityActions( void );

    //! navigation actions
    void _updateNavigationActions( void );

    //! update active frame
    void _updateActiveFrame( QWidget*, QWidget* );

    //! update view mode
    void _updateViewMode( FileSelectionFrame::ViewMode );

    //! update places items
    void _updatePlacesItems( void );

    //! update places items
    void _updatePlacesItems( bool value );

    //! file selected in one of the selection frame
    void _fileSelected( const FileInfo& );

    //! file selected in one of the selection frame
    void _placeSelected( const BaseFileInfo& );

    //! connect
    void _connect( void );

    //! reload current directory
    void _reload( void );

    //! select previous directory from history
    void _previousDirectory( void );

    //! select next directory from history
    void _nextDirectory( void );

    //! select parent directory
    void _parentDirectory( void );

    //! select home directory
    void _homeDirectory( void );

    //! update clipboard
    void _updateClipboard( const FileSelectionClipboard& = FileSelectionClipboard() );

    //! paste (using drag and drop)
    void _pasteSelection( const FileInfo&, const FileInfo::List& );

    //! paste clipboard to path in argument
    void _pasteClipboard( const FileInfo& );

    //! open files
    void _openFiles( const FileInfo::List& );

    //! find collection
    void _findCollection( const FileInfo& );

    //! show file information
    void _fileInformation( const FileInfo&, const QIcon& );

    //! toggle show navigator
    void _toggleShowNavigator( bool );

    //! toggle show hidden files
    void _toggleShowHiddenFiles( bool );

    //! show command list
    void _showCommandDialog( void );

    //! show log
    void _showLog( void );

    //! toggle log
    void _toggleLog( bool );

    //! log dialog hidden
    void _logDialogHidden( void );

    //! command list destroyed
    void _commandDialogHidden( void );

    private slots:

    //! configuration
    void _updateConfiguration( void );

    private:

    //! install actions
    void _installActions( void );

    //! icon provider
    FileIconProvider* iconProvider_;

    //! local files
    FileSelectionFrame* localSelectionFrame_;

    //! remote files
    FileSelectionFrame* remoteSelectionFrame_;

    //! clipboard (for copy/paste actions)
    FileSelectionClipboard clipboard_;

    //! file system watcher, for local selection frame
    FileSystemWatcher fileSystemWatcher_;

    //! statusbar
    ProgressStatusBar* statusbar_;

    //! command list
    AlienCommandDialog* commandDialog_;

    //! log dialog
    LogDialog* logDialog_;

    //! alien interface
    AlienInterface* interface_;

    //! busy widget
    BusyWidget* busyWidget_;

    //! places
    PlacesWidget* placesWidget_;

    //! dock widget
    DockWidget* placesPanel_;

    //!@name actions
    //@{

    //! connection
    QAction* connectAction_;

    //! home directory action
    QAction* homeDirectoryAction_;

    //! previous directory action
    QAction* previousDirectoryAction_;

    //! next directory action
    QAction* nextDirectoryAction_;

    //! parent directory action
    QAction* parentDirectoryAction_;

    //! reload
    QAction* reloadAction_;

    //! abort
    QAction* abortAction_;

    //! hide hidden files
    QAction* showHiddenFilesAction_;

    //! static menu actions
    QList<QAction*> menuActions_;

    //! command list
    QAction* commandDialogAction_;

    //! log action
    QAction* logAction_;

    //! log action
    QAction* toggleLogAction_;

    //@}

    //! show navigator
    bool showNavigator_;

    //! true if connection was valid at least once
    bool wasConnected_;

};

#endif

